# pythran export _brief_loop(float64[:,:], uint8[:,:],
#                            intp[:,:], int[:,:], int[:,:])


def _brief_loop(image, descriptors, keypoints, pos0, pos1):
    for k in range(len(keypoints)):
        kr, kc = keypoints[k]
        for p in range(len(pos0)):
            pr0, pc0 = pos0[p]
            pr1, pc1 = pos1[p]
            descriptors[k, p] = (image[kr + pr0, kc + pc0]
                                 < image[kr + pr1, kc + pc1])



#pythran export get_primes7(int)
#from http://blog.famzah.net/2010/07/01/cpp-vs-python-vs-perl-vs-php-performance-benchmark/
#runas get_primes7(100)
#bench get_primes7(7000000)
def get_primes7(n):
	"""
	standard optimized sieve algorithm to get a list of prime numbers
	--- this is the function to compare your functions against! ---
	"""
	if n < 2:  return []
	if n == 2: return [2]
	# do only odd numbers starting at 3
	s = list(range(3, n+1, 2))
	# n**0.5 simpler than math.sqr(n)
	mroot = n ** 0.5
	half = len(s)
	i = 0
	m = 3
	while m <= mroot:
		if s[i]:
			j = (m*m-3)//2  # int div
			s[j] = 0
			while j < half:
				s[j] = 0
				j += m
		i = i+1
		m = 2*i+3
	return [2]+[x for x in s if x]



# Author: Robert T. McGibbon, December 2014

from numpy import zeros, asarray, complex128, float64, zeros_like

#pythran export levinson(float64[], float64[])
#pythran export levinson(complex128[], complex128[])
#runas import numpy as np; x = np.arange(1, 4.)*7; y = np.arange(-11., -9.)

def levinson(a, b):
    """Solve a linear Toeplitz system using Levinson recursion.

    Parameters
    ----------
    a : array, dtype=double or complex128, shape=(2n-1,)
        The first column of the matrix in reverse order (without the diagonal)
        followed by the first (see below)
    b : array, dtype=double  or complex128, shape=(n,)
        The right hand side vector. Both a and b must have the same type
        (double or complex128).

    Notes
    -----
    For example, the 5x5 toeplitz matrix below should be represented as
    the linear array ``a`` on the right ::

        [ a0    a1   a2  a3  a4 ]
        [ a-1   a0   a1  a2  a3 ]
        [ a-2  a-1   a0  a1  a2 ] -> [a-4  a-3  a-2  a-1  a0  a1  a2  a3  a4]
        [ a-3  a-2  a-1  a0  a1 ]
        [ a-4  a-3  a-2  a-1 a0 ]

    Returns
    -------
    x : arrray, shape=(n,)
        The solution vector
    reflection_coeff : array, shape=(n+1,)
        Toeplitz reflection coefficients. When a is symmetric Toeplitz and
        ``b`` is ``a[n:]``, as in the solution of autoregressive systems,
        then ``reflection_coeff`` also correspond to the partial
        autocorrelation function.
    """
    # Adapted from toeplitz.f90 by Alan Miller, accessed at
    # http://jblevins.org/mirror/amiller/toeplitz.f90
    # Released under a Public domain declaration.

    n = b.shape[0]
    x = zeros_like(b)  # result
    g = zeros_like(b)  # workspace
    h = zeros_like(b)  # workspace
    reflection_coeff = zeros(n+1, dtype=b.dtype)  # history
    assert len(a) == (2*n) - 1

    if a[n-1] == 0:
        raise ValueError('Singular principal minor')

    x[0] = b[0] / a[n-1]
    reflection_coeff[0] = 1
    reflection_coeff[1] = x[0]

    if (n == 1):
        return asarray(x), asarray(reflection_coeff)

    g[0] = a[n-2] / a[n-1]
    h[0] = a[n] / a[n-1]

    for m in range(1, n):
        # Compute numerator and denominator of x[m]
        x_num = -b[m]
        x_den = -a[n-1]
        for j in range(m):
            nmj = n + m - (j+1)
            x_num = x_num + a[nmj] * x[j]
            x_den = x_den + a[nmj] * g[m-j-1]
        if x_den == 0:
            raise ValueError('Singular principal minor')
        x[m] = x_num / x_den
        reflection_coeff[m+1] = x[m]

        # Compute x
        for j in range(m):
            x[j] = x[j] - x[m] * g[m-j-1]
        if m == n-1:
            return asarray(x), asarray(reflection_coeff)

        # Compute the numerator and denominator of g[m] and h[m]
        g_num = -a[n-m-2]
        h_num = -a[n+m]
        g_den = -a[n-1]
        for j in range(m):
            g_num = g_num + a[n+j-m-1] * g[j]
            h_num = h_num + a[n+m-j-1] * h[j]
            g_den = g_den + a[n+j-m-1] * h[m-j-1]

        if g_den == 0.0:
            raise ValueError("Singular principal minor")

        # Compute g and h
        g[m] = g_num / g_den
        h[m] = h_num / x_den
        k = m - 1
        m2 = (m + 1) >> 1
        c1 = g[m]
        c2 = h[m]
        for j in range(m2):
            gj = g[j]
            gk = g[k]
            hj = h[j]
            hk = h[k]
            g[j] = gj - (c1 * hk)
            g[k] = gk - (c1 * hj)
            h[j] = hj - (c2 * gk)
            h[k] = hk - (c2 * gj)
            k -= 1



from distutils.core import setup
from pythran.dist import PythranExtension, PythranBuildExt

module1 = PythranExtension('demo', sources = ['a.py'])

setup(name = 'demo',
      version = '1.0',
      description = 'This is a demo package',
      cmdclass={"build_ext": PythranBuildExt},
      ext_modules = [module1])



#runas solve()
#unittest.skip recursive generator
#pythran export solve()
''' From O'Reilly's Python Cookbook '''

def _combinators(_handle, items, n):
    if n==0:
        yield []
        return
    for i, item in enumerate(items):
        this_one = [ item ]
        for cc in _combinators(_handle, _handle(items, i), n-1):
            yield this_one + cc

def combinations(items, n):
    ''' take n distinct items, order matters '''
    def skipIthItem(items, i):
        return items[:i] + items[i+1:]
    return _combinators(skipIthItem, items, n)

def uniqueCombinations(items, n):
    ''' take n distinct items, order is irrelevant '''
    def afterIthItem(items, i):
        return items[i+1:]
    return _combinators(afterIthItem, items, n)

def selections(items, n):
    ''' take n (not necessarily distinct) items, order matters '''
    def keepAllItems(items, i):
        return items
    return _combinators(keepAllItems, items, n)

def permutations(items):
    ''' take all items, order matters '''
    return combinations(items, len(items))
def solve():
    '''
    We shall say that an n-digit number is pandigital if it makes use of all the digits 1 to n exactly once. For example, 2143 is a 4-digit pandigital and is also prime.
    
    What is the largest n-digit pandigital prime that exists?
    '''

    prime_list = [2, 3, 5, 7, 11, 13, 17, 19, 23]   # Ensure that this is initialised with at least 1 prime
    prime_dict = dict.fromkeys(prime_list, 1)

    def _isprime(n):
        ''' Raw check to see if n is prime. Assumes that prime_list is already populated '''
        isprime = n >= 2 and 1 or 0
        for prime in prime_list:                    # Check for factors with all primes
            if prime * prime > n: break             # ... up to sqrt(n)
            if not n % prime:
                isprime = 0
                break
        if isprime: prime_dict[n] = 1               # Maintain a dictionary for fast lookup
        return isprime

    def _refresh(x):
        ''' Refreshes primes upto x '''
        lastn = prime_list[-1]
        while lastn <= x:                           # Keep working until we've got up to x
            lastn = lastn + 1                       # Check the next number
            if _isprime(lastn):
                prime_list.append(lastn)            # Maintain a list for sequential access



    # Pan-digital primes are 4 or 7 digits. Others divisible by 3
    _refresh(2766)    # sqrt(7654321)
    for perm in permutations(range(7, 0, -1)):
        num = 0
        for n in perm: num = num * 10 + n
        if _isprime(num):
            return num
            break




#pythran export CCOPY(int, complex list, int, complex list, int)
#runas CCOPY(2,[complex(1,2),complex(2,3),complex(3,4),complex(5,6)],2,[complex(3,4),complex(1,2),complex(2,3),complex(5,6)],3)
#bench sz = 20000000; in1 = map(complex, range(sz), range(sz)); in2 = map(complex, range(sz), range(sz));CCOPY(sz / 6,in1,2,in2,3)
def CCOPY(N,CX,INCX,CY,INCY):
#  Purpose
#  =======
#
#     CCOPY copies a vector x to a vector y.
#
#  Further Details
#  ===============
#
#
#  =====================================================================
#
    if N <= 0:
        return
    if (INCX==1 and INCY==1):
#
#        code for both increments equal to 1
#
        for I in range(N):
            CY[I] = CX[I]
    else:
#
#        code for unequal increments or equal increments
#          not equal to 1
#
        IX = 0
        IY = 0
        if (INCX < 0):
            IX = (-N+1)*INCX
        if (INCY < 0):
            IY = (-N+1)*INCY
        for I in range(N):
            CY[IY] = CX[IX]
            IX = IX + INCX
            IY = IY + INCY
    return



from pythran.tests import TestEnv
from pythran.typing import Dict, List

class TestDict(TestEnv):

    def test_dict_(self):
        self.run_test("def dict_(): a=dict()", dict_=[])

    def test_assigned_dict(self):
        self.run_test("def assigned_dict(k):\n a=dict() ; a[k]=18", "yeah", assigned_dict=[str])

    def test_print_empty_dict(self):
        self.run_test("def print_empty_dict():\n print(dict())", print_empty_dict=[])

    def test_print_dict(self):
        self.run_test("def print_dict(k):\n a= dict() ; a[k]='youpi'\n print(a)", 5, print_dict=[int])

    def test_empty_dict(self):
        self.run_test("def empty_dict(): return {}", empty_dict=[])

    def test_initialized_dict(self):
        self.run_test("def initialized_dict(): return {1:'e', 5.2:'f'}", initialized_dict=[])

    def test_dict_contains(self):
        self.run_test("def dict_contains(v): return v in { 'a':1, 'e': 2 }", "e", dict_contains=[str])

    def test_emptydict_contains(self):
        self.run_test("def emptydict_contains(v): return v in dict()", "e", emptydict_contains=[str])

    def test_dict_get_item(self):
        self.run_test("def dict_get_item(a): return a['e']", {'e':1, 'f':2}, dict_get_item=[Dict[str, int]])

    def test_dict_len(self):
        self.run_test("def dict_len(d): return len(d)", {1:'e', 2:'f'}, dict_len=[Dict[int, str]])
    def test_dict_set_item(self):
        self.run_test("def dict_set_item():\n a= dict() ; a[1.5]='s'\n return a", dict_set_item=[])
    def test_dict_set_item_bis(self):
        self.run_test("def dict_set_item_bis():\n a= dict() ; a[1]='s'\n return a", dict_set_item_bis=[])
    def test_dict_clear(self):
        self.run_test("def dict_clear(a):\n a.clear()\n return a", {'e':'E' }, dict_clear=[Dict[str, str]])
    def test_dict_copy(self):
        code="""
def dict_copy(a):
    b = a.copy()
    c = a
    a.clear()
    return c,b"""
        self.run_test(code,  {1:2 }, dict_copy=[Dict[int, int]])

    def test_dict_from_keys(self):
        return self.run_test("def dict_from_keys(a): return dict.fromkeys(a), dict.fromkeys(a,1)", [1.5,2.5,3.5], dict_from_keys=[List[float]])

    def test_dict_get(self):
        return self.run_test("def dict_get(a): return a.get(1.5), a.get(2, 18) + 1", {1.5:2 }, dict_get=[Dict[float, int]])

    def test_dict_get_none(self):
        return self.run_test("def dict_get_none(a): return a.get(1)", {1.5:2 }, dict_get_none=[Dict[float, int]])

    def test_dict_items(self):
        return self.run_test("def dict_items(a): return sorted(a.items())", { 'a':1, 'e': 2 }, dict_items=[Dict[str, int]])

    def test_dict_for(self):
        return self.run_test("def dict_for(a): return sorted([x for x in a])", { 'a':1, 'e': 2 }, dict_for=[Dict[str, int]])

    def test_dict_keys(self):
        return self.run_test("def dict_keys(a): return sorted([ x*2 for x in a.keys()])", { 1:'a', 2:'b' }, dict_keys=[Dict[int, str]])

    def test_dict_values(self):
        return self.run_test("def dict_values(a): return sorted([ x*2 for x in a.values()])", { 1:'a', 2:'b' }, dict_values=[Dict[int, str]])

    def test_dict_pop(self):
        return self.run_test("def dict_pop(a): return a.pop(1), a.pop(3,'e'), a", { 1:'a', 2:'b' }, dict_pop=[Dict[int, str]])

    def test_dict_popitem0(self):
        return self.run_test(
            "def dict_popitem0(a): return a.popitem(), a",
            {1:'a' },
            dict_popitem0=[Dict[int, str]])

    def test_dict_popitem1(self):
        return self.run_test(
            "def dict_popitem1(a): return a.popitem(), a",
            { 1: 2 },
            dict_popitem1=[Dict[int, int]])

    def test_dict_setdefault(self):
        return self.run_test("def dict_setdefault():\n a={1.5:2 }\n return a.setdefault(1.5) + a.setdefault(2, 18)", dict_setdefault=[])

    def test_dict_update(self):
        return self.run_test("def dict_update(a):\n a.update([(1,'e')])\n a.update({2:'c'})\n return a", { 1:'a', 2:'b' }, dict_update=[Dict[int, str]])

    def test_dict_items_contains(self):
        return self.run_test("def dict_viewitems_contains(a):\n d=a.items()\n return (1,'a') in d, (2,'e') in d", { 1:'a', 2:'b' }, dict_viewitems_contains=[Dict[int, str]])

    def test_dict_keys_contains(self):
        return self.run_test("def dict_viewkeys_contains(a):\n d=a.keys()\n return 1 in d, 3 in d", { 1:'a', 2:'b' }, dict_viewkeys_contains=[Dict[int, str]])

    def test_dict_values_contains(self):
        return self.run_test("def dict_viewvalues_contains(a):\n d=a.values()\n return 'a' in d, 'e' in d", { 1:'a', 2:'b' }, dict_viewvalues_contains=[Dict[int, str]])

    def test_dict_update_combiner(self):
        return self.run_test("def dict_update_combiner():\n a=dict()\n a.update({1:'e'})\n return a", dict_update_combiner=[])

    def test_dict_setdefault_combiner(self):
        return self.run_test("def dict_setdefault_combiner():\n a=dict()\n a.setdefault(1,'e')\n return a", dict_setdefault_combiner=[])

    def test_dict_iterate_item(self):
        return self.run_test(
            """def dict_iterate_item(d):
                s = 0
                for kv in d.items():
                 for e in kv:
                  s += e
                return s""",
            {1:2,3:4},
            dict_iterate_item=[Dict[int, int]])



from pythran.tests import TestEnv
from pythran.typing import List

import unittest


class TestStr(TestEnv):
    def test_str_startswith0(self):
        self.run_test("def str_startswith0(s0, s1): return s0.startswith(s1)", "barbapapa", "barba", str_startswith0=[str, str])
        self.run_test("def str_startswith0a(s0, s1): return s0[0].startswith(s1)", "barbapapa", "barba", str_startswith0a=[str, str])

    def test_str_startswith1(self):
        self.run_test("def str_startswith1(s0, s1): return s0.startswith(s1)", "barbapapa", "barbi", str_startswith1=[str, str])
        self.run_test("def str_startswith1a(s0, s1): return s0[0].startswith(s1)", "barbapapa", "barbi", str_startswith1a=[str, str])

    def test_str_endswith0(self):
        self.run_test("def str_endswith0(s0, s1): return s0.endswith(s1)", "barbapapa", "papa", str_endswith0=[str, str])
        self.run_test("def str_endswith0a(s0, s1): return s0[0].endswith(s1)", "barbapapa", "papa", str_endswith0a=[str, str])

    def test_str_endswith1(self):
        self.run_test("def str_endswith1(s0, s1): return s0.endswith(s1)", "barbapapa", "papy", str_endswith1=[str, str])
        self.run_test("def str_endswith1a(s0, s1): return s0[0].endswith(s1)", "barbapapa", "papy", str_endswith1a=[str, str])

    def test_str_empty(self):
        self.run_test("def str_empty(s0): return '>o_/' if s0 else '0x0'", "", str_empty=[str])

    def test_str_failed_conversion(self):
        self.run_test("def str_failed_conversion(s):\n try: return int(s)\n except: return 42", "prout", str_failed_conversion=[str])
        self.run_test("def str_failed_conversion_a(s):\n try: return int(s[0])\n except: return 42", "prout", str_failed_conversion_a=[str])

    def test_str_replace0(self):
        self.run_test("def str_replace0(s): return s.replace('er', 'rer')", "parler", str_replace0=[str])
        self.run_test("def str_replace0a(s): return s.replace('er', 'r')", "parler", str_replace0a=[str])
        self.run_test("def str_replace0b(s): return s[0].replace('er', 'r')", "parler", str_replace0b=[str])

    def test_str_replace1(self):
        self.run_test("def str_replace1(s): return s.replace('er', 'rer', 1)", "erlang manger dessert", str_replace1=[str])
        self.run_test("def str_replace1a(s): return s.replace('er', 'r', 1)", "erlang manger dessert", str_replace1a=[str])
        self.run_test("def str_replace1b(s): return s[0].replace('er', 'r', 1)", "erlang manger dessert", str_replace1b=[str])

    def test_str_replace2(self):
        self.run_test("def str_replace2(s): return s.replace('', 'du vide surgit rien', 1)", "j aime les moulinettes a fromage", str_replace2=[str])
        self.run_test("def str_replace2a(s): return s[0].replace('', 'du vide surgit rien', 1)", "j aime les moulinettes a fromage", str_replace2a=[str])

    def test_str_ascii_letters(self):
        self.run_test("def str_ascii_letters(): import string; return string.ascii_letters", str_ascii_letters=[])

    def test_str_ascii_lowercase(self):
        self.run_test("def str_ascii_lowercase(): import string; return string.ascii_lowercase", str_ascii_lowercase=[])

    def test_str_ascii_uppercase(self):
        self.run_test("def str_ascii_uppercase(): import string; return string.ascii_uppercase", str_ascii_uppercase=[])

    def test_str_digits(self):
        self.run_test("def str_digits(): import string; return string.digits", str_digits=[])

    def test_str_hexdigits(self):
        self.run_test("def str_hexdigits(): import string; return string.hexdigits", str_hexdigits=[])

    def test_str_octdigits(self):
        self.run_test("def str_octdigits(): import string; return string.octdigits", str_octdigits=[])

    def test_str_lower(self):
        self.run_test("def str_lower(s): return s.lower()", "ThiS iS a TeST", str_lower=[str])
        self.run_test("def str_lower_a(s): return s[0].lower()", "ThiS iS a TeST", str_lower_a=[str])

    def test_str_upper(self):
        self.run_test("def str_upper(s): return s.upper()", "ThiS iS a TeST", str_upper=[str])
        self.run_test("def str_upper_a(s): return s[0].upper()", "ThiS iS a TeST", str_upper_a=[str])

    def test_str_capitalize(self):
        self.run_test("def str_capitalize(s): return s.capitalize()", "thiS iS a TeST", str_capitalize=[str])
        self.run_test("def str_capitalize_a(s): return s[0].capitalize()", "thiS iS a TeST", str_capitalize_a=[str])

    def test_str_strip(self):
        self.run_test("def str_strip(s): return s.strip()", "       ThiS iS a TeST        ", str_strip=[str])
        self.run_test("def str_strip_a(s): return s[0].strip()", "       ThiS iS a TeST        ", str_strip_a=[str])

    def test_str_strip2(self):
        self.run_test("def str_strip2(s): return s.strip(\"TSih\")", "ThiS iS a TeST", str_strip2=[str])

    def test_str_lstrip(self):
        self.run_test("def str_lstrip(s): return s.lstrip()", "       ThiS iS a TeST        ", str_lstrip=[str])
        self.run_test("def str_lstrip_a(s): return s[0].lstrip()", "       ThiS iS a TeST        ", str_lstrip_a=[str])

    def test_str_lstrip2(self):
        self.run_test("def str_lstrip2(s): return s.lstrip(\"TSih\")", "ThiS iS a TeST", str_lstrip2=[str])

    def test_str_rstrip(self):
        self.run_test("def str_rstrip(s): return s.rstrip()", "       ThiS iS a TeST        ", str_rstrip=[str])
        self.run_test("def str_rstrip_a(s): return s[0].rstrip()", "       ThiS iS a TeST        ", str_rstrip_a=[str])

    def test_str_rstrip2(self):
        self.run_test("def str_rstrip2(s): return s.rstrip(\"TSih\")", "ThiS iS a TeST", str_rstrip2=[str])

    def test_str_split1(self):
        self.run_test("def str_split1(s): return s.split()", "ThiS iS a TeST", str_split1=[str])
        self.run_test("def str_split1_a(s): return s[0].split()", "ThiS iS a TeST", str_split1_a=[str])

    def test_str_split2(self):
        self.run_test("def str_split2(s): return s.split('S')", "ThiS iS a TeST", str_split2=[str])

    def test_str_split3(self):
        self.run_test("def str_split3(s): return s.split()", "ThiS  iS\t a TeST", str_split3=[str])

    def test_str_split4(self):
        self.run_test("def str_split4(s): return s.split()", "", str_split4=[str])

    def test_str_format(self):
        self.run_test("def str_format(a): return '%.2f %.2f' % (a, a)", 43.23, str_format=[float])

    def test_str_join0(self):
        self.run_test("def str_join0(): a = ['1'] ; a.pop() ; return '+e+'.join(a)", str_join0=[])
        self.run_test("def str_join0a(): a = ['1'] ; a.pop() ; return 'e'.join(a)", str_join0a=[])

    def test_str_join1(self):
        self.run_test("def str_join1(): a = ['l', 'l'] ; return 'o'.join(a)", str_join1=[])

    def test_str_join2(self):
        self.run_test("def str_join2(a): return 'o'.join(filter(len, a))", ['l', 'l'], str_join2=[List[str]])

    def test_str_find0(self):
        self.run_test("def str_find0(s): return s.find('pop')", "popop", str_find0=[str])
        self.run_test("def str_find0a(s): return s.find('p')", "popop", str_find0a=[str])
        self.run_test("def str_find0b(s): return s[0].find('p')", "popop", str_find0b=[str])

    def test_str_find1(self):
        self.run_test("def str_find1(s): return s.find('pap')", "popop", str_find1=[str])

    def test_str_reversal(self):
        self.run_test("def str_reversal(s): return list(map(ord,reversed(s)))", "dear", str_reversal=[str])
        self.run_test("def str_reversal_a(s): return list(map(ord,reversed(s[0])))", "dear", str_reversal_a=[str])

    def test_str_substring_iteration(self):
        self.run_test("def str_substring_iteration(s): return list(map(ord, s[1:-1]))", "pythran", str_substring_iteration=[str])

    def test_str_isalpha(self):
        self.run_test("def str_isalpha(s, t, u): return s.isalpha(), t.isalpha(), u.isalpha()", "e", "1", "", str_isalpha=[str,str, str])
        self.run_test("def str_isalpha_a(s, t): return s[0].isalpha(), t[0].isalpha()", "e", "1", str_isalpha_a=[str,str])

    def test_str_isdigit(self):
        self.run_test("def str_isdigit(s, t, u): return s.isdigit(), t.isdigit(), u.isdigit()", "e", "1", "", str_isdigit=[str,str, str])
        self.run_test("def str_isdigit_a(s, t): return s[0].isdigit(), t[0].isdigit()", "e", "1", str_isdigit_a=[str,str])

    def test_str_count(self):
        self.run_test("def str_count(s, t, u, v): return s.count(t), s.count(u), s.count(v)",
                      "pythran is good for health", "py", "niet", "t",
                      str_count=[str, str, str, str])
        self.run_test("def str_count_a(s, t, u, v): return s[0].count(t), s[1].count(u), s.count(v[0])",
                      "pythran is good for health", "py", "niet", "t",
                      str_count_a=[str, str, str, str])

    def test_str_literal_cmp(self):
        code = '''
            def eee(a, i):
                if a == "ABCD":
                    return 2 * i
                elif a != "ZDSD":
                    return 1 * i
                return 3 * i

            def str_literal_cmp(a, i):
                if a == "EEE":
                   return eee("ZZZ", i), eee("ABCD", i)
                else:
                   return eee("YYY", i), 3'''
        self.run_test(code, "EEE", 2, str_literal_cmp=[str, int])

    def test_str_literal_cmp1(self):
        code = '''
            def eee(a, i):
                if a > "ABCD":
                    return 2 * i
                elif a <= "ZDSD":
                    return 1 * i
                return 3 * i

            def str_literal_cmp1(a, i):
                if a == "EEE":
                   return eee("ZZZ", i), eee("ABCD", i)
                else:
                   return eee("YYY", i), 3'''
        self.run_test(code, "EEE", 2, str_literal_cmp1=[str, int])

    def test_str_literal_cmp2(self):
        code = '''
            def eee(a, i):
                if a < "ABCD":
                    return 2 * i
                elif a >= "ZDSD":
                    return 1 * i
                return 3 * i

            def str_literal_cmp2(a, i):
                if a == "EEE":
                   return eee("ZZZ", i), eee("ABCD", i)
                else:
                   return eee("YYY", i), 3'''
        self.run_test(code, "EEE", 2, str_literal_cmp2=[str, int])

    def test_str_literal_add(self):
        code = '''
            def eee(a, i):
                if i > 0:
                    return a + "ABCD"
                else:
                    return a + "BCD"

            def str_literal_add(a, i):
                if a == "EEE":
                   return eee("ZZZ", i), eee("ABCD", i)
                else:
                   return eee("YYY", i), "3"'''
        self.run_test(code, "EEE", 2, str_literal_add=[str, int])

    def test_str_literal_mult(self):
        code = '''
            def eee(a, i):
                if i > 0:
                    return a * i
                else:
                    return a * 3

            def str_literal_mult(a, i):
                if a == "EEE":
                   return eee("ZZZ", i), eee("ABCD", i)
                else:
                   return eee("YYY", i), "3"'''
        self.run_test(code, "EEE", 2, str_literal_mult=[str, int])

    def test_str_float(self):
        self.run_test("def str_float(s): return float(s)", "0.000012", str_float=[str])

    def test_str_numpy_float32(self):
        self.run_test("def str_numpy_float32(s): import numpy; return numpy.float32(s)", "0.000012",
                      str_numpy_float32=[str])

    def test_str_numpy_float64(self):
        self.run_test("def str_numpy_float64(s): import numpy; return numpy.float64(s)", "0.000012",
                      str_numpy_float64=[str])

    def test_str_int(self):
        self.run_test("def str_int(s): return int(s)", "12", str_int=[str])

    def test_str_id(self):
        self.run_test("def str_id(x): return id(x) != 0", "hello", str_id=[str])

    def test_str_slice_assign(self):
        self.run_test('''
            def str_slice_assign(s, c):
                if s.startswith(c):
                    s = s[len(c):];
                return s''', "LEFT-B6", "LEFT-",
                      str_slice_assign=[str, str])

    def test_str_slice_assign2(self):
        self.run_test('''
        def sample_datatype(value):
            definitions = [ 
                ('LEFT-', 1), 
                ('RIGHT-', 2), 
                ('', 3)
            ]   
        
            plate_number = None
            for definition in definitions:
                s, n = definition
                if value.startswith(s):
                    plate_number = n 
                    value = value[len(s):]
                    break
            if plate_number is None:   # Comment for make it works
               raise ValueError("Invalid value")
            return None

        def str_slice_assign2(s1):
            sample_datatype(s1)
            return s1''', "LEFT-B6", str_slice_assign2=[str])



#imported from https://bitbucket.org/pypy/benchmarks/src/846fa56a282b0e8716309f891553e0af542d8800/own/fannkuch.py?at=default
# the export line is in fannkuch.pythran
#runas fannkuch(9);fannkuch2(9)
#bench fannkuch(9)

def fannkuch(n):
    count = list(range(1, n+1))
    max_flips = 0
    m = n-1
    r = n
    check = 0
    perm1 = list(range(n))
    perm = list(range(n))

    while 1:
        if check < 30:
            #print("".join(str(i+1) for i in perm1))
            check += 1

        while r != 1:
            count[r-1] = r
            r -= 1

        if perm1[0] != 0 and perm1[m] != m:
            perm = perm1[:]
            flips_count = 0
            k = perm[0]
            while k:
                perm[:k+1] = perm[k::-1]
                flips_count += 1
                k = perm[0]

            if flips_count > max_flips:
                max_flips = flips_count

        while r != n:
            perm1.insert(r, perm1.pop(0))
            count[r] -= 1
            if count[r] > 0:
                break
            r += 1
        else:
            return max_flips

def fannkuch2(n):
    fannkuch(n)



#from http://rosettacode.org/wiki/Happy_numbers#Python
#pythran export happy(int)
#runas [x for x in range(500) if happy(x)][:8]

def happy(n):
    past = set()
    while n != 1:
        n = sum(int(i)**2 for i in str(n))
        if n in past:
            return False
        past.add(n)
    return True



from __future__ import print_function

# Preliminary checks that cannot be done by setuptools
# like... the setuptools dependency itself!
try:
    import setuptools
except ImportError:
    print()
    print("*****************************************************")
    print("* Setuptools must be installed before running setup *")
    print("*****************************************************")
    print()
    raise

from setuptools.command.build_py import build_py
from setuptools.command.develop import develop

from setuptools import setup

import logging
import os
import shutil
import sys

# It appears old versions of setuptools are not supported, see
#   https://github.com/serge-sans-paille/pythran/issues/489

from distutils.version import LooseVersion
MinimalSetuptoolsVersion = LooseVersion("12.0.5")
if LooseVersion(setuptools.__version__) < MinimalSetuptoolsVersion:
    msg = "Setuptools version is {}, but must be at least {}".format(
        setuptools.__version__,
        MinimalSetuptoolsVersion)
    print()
    print("*" * (len(msg) + 4))
    print("*", msg, "*")
    print("*" * (len(msg) + 4))
    print()
    raise ImportError("setuptools")

if sys.version_info.major < 3:
    print()
    print("****************************************************")
    print("*        Python 2 has reached end-of-support       *")
    print("****************************************************")
    print("*                                                  *")
    print("* Last Pythran version supporting Python2 is 0.9.5 *")
    print("*                                                  *")
    print("****************************************************")
    print()


logger = logging.getLogger("pythran")
logger.addHandler(logging.StreamHandler())

versionfile = os.path.join('pythran', 'version.py')
exec(open(versionfile).read())


class BuildWithThirdParty(build_py):

    """
    Set up Pythran dependencies.

    * install boost dependencies
    * install xsimd dependencies
    """

    third_parties = 'boost', 'xsimd'

    user_options = build_py.user_options + [
        ('no-{}'.format(pkg), None, 'Do not distribute {} headers'.format(pkg))
        for pkg in third_parties
    ]

    def initialize_options(self):
        build_py.initialize_options(self)
        for pkg in BuildWithThirdParty.third_parties:
            setattr(self, 'no_' + pkg, None)

    def copy_pkg(self, pkg, src_only=False):
        "Install boost deps from the third_party directory"

        if getattr(self, 'no_' + pkg) is None:
            print('Copying', pkg, 'dependencies')
            to_copy = pkg,
        else:
            return

        src = os.path.join('third_party', *to_copy)

        # copy to the build tree
        if not src_only:
            target = os.path.join(self.build_lib, 'pythran', *to_copy)
            shutil.rmtree(target, True)
            shutil.copytree(src, target)

        # copy them to the source tree too, needed for sdist
        target = os.path.join('pythran', *to_copy)
        shutil.rmtree(target, True)
        shutil.copytree(src, target)

    def run(self, *args, **kwargs):
        # regular build done by parent class
        build_py.run(self, *args, **kwargs)
        if not self.dry_run:  # compatibility with the parent options
            for pkg in BuildWithThirdParty.third_parties:
                self.copy_pkg(pkg)


class DevelopWithThirdParty(develop, BuildWithThirdParty):

    def initialize_options(self):
        develop.initialize_options(self)
        BuildWithThirdParty.initialize_options(self)

    def run(self, *args, **kwargs):
        if not self.dry_run:  # compatibility with the parent options
            for pkg in BuildWithThirdParty.third_parties:
                self.copy_pkg(pkg, src_only=True)
        develop.run(self, *args, **kwargs)


# Cannot use glob here, as the files may not be generated yet
boost_headers = ['boost/' + '*/' * i + '*.hpp' for i in range(1, 20)]
xsimd_headers = ['xsimd/' + '*/' * i + '*.hpp' for i in range(1, 20)]
pythonic_headers = ['*/' * i + '*.hpp' for i in range(9)] + ['patch/*']


# read longdescr from README
def longdescr(readme_path):
    with open(readme_path) as readme:
        lines = list(readme)
        start_index = lines.index('What is it?\n')
        stop_index = lines.index('Installation\n')
        long_description = "".join(lines[start_index + 2: stop_index])
        return long_description


setup(name='pythran',
      version=__version__,
      description=__descr__,
      long_description=longdescr("README.rst"),
      author='Serge Guelton',
      author_email='serge.guelton@telecom-bretagne.eu',
      url=__url__,
      packages=['pythran', 'pythran.analyses', 'pythran.transformations',
                'pythran.optimizations', 'omp', 'pythran/pythonic',
                'pythran.types'],
      package_data={'pythran': (['pythran*.cfg']
                                + boost_headers
                                + xsimd_headers),
                    'pythran/pythonic': pythonic_headers},
      classifiers=[
          'Development Status :: 4 - Beta',
          'Environment :: Console',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: BSD License',
          'Natural Language :: English',
          'Operating System :: POSIX :: Linux',
          'Operating System :: MacOS',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: Implementation :: CPython',
          'Programming Language :: C++',
          'Topic :: Software Development :: Compilers',
          'Topic :: Software Development :: Code Generators'
      ],
      license="BSD 3-Clause",
      install_requires=open('requirements.txt').read().splitlines(),
      entry_points={'console_scripts':
                    ['pythran = pythran.run:run',
                     'pythran-config = pythran.config:run']},
      extras_require={
          'doc': open('docs/requirements.txt').read().splitlines(),
      },
      test_suite="pythran.tests.test_cases",
      cmdclass={'build_py': BuildWithThirdParty,
                'develop': DevelopWithThirdParty})



#from http://rosettacode.org/wiki/Rep-string#Python
#pythran export is_repeated(str)
#pythran export reps(str)
#runas matchstr ="1001110011 1110111011 0010010010 1010101010 1111111111 0100101101 0100100 101 11 00 1"; [reps(i) for i in matchstr.split()]
#runas matchstr ="1001110011 1110111011 0010010010 1010101010 1111111111 0100101101 0100100 101 11 00 1"; [is_repeated(i) for i in matchstr.split()]

def is_repeated(text):
    'check if the first part of the string is repeated throughout the string'
    for x in range(len(text)//2, 0, -1):
        if text.startswith(text[x:]): return x
    return 0

def reps(text):
    return [text[:x] for x in range(1, 1 + len(text) // 2)
            if text.startswith(text[x:])]



import unittest
from pythran.tests import TestEnv
import numpy
import tempfile
import os

from pythran.typing import NDArray, List, Tuple


@TestEnv.module
class TestNumpyFunc0(TestEnv):

    def test_extended_sum0(self):
        self.run_test("def numpy_extended_sum0(a): import numpy ; return numpy.sum(a)",
                      numpy.arange(120).reshape((3,5,4,2)),
                      numpy_extended_sum0=[NDArray[int,:,:,:,:]])

    def test_extended_sum1(self):
        self.run_test("def numpy_extended_sum1(a): import numpy ; return numpy.sum(a[1])",
                      numpy.arange(120).reshape((3,5,4,2)),
                      numpy_extended_sum1=[NDArray[int,:,:,:,:]])

    def test_extended_sum2(self):
        self.run_test("def numpy_extended_sum2(a): import numpy ; return numpy.sum(a[1,0])",
                      numpy.arange(120).reshape((3,5,4,2)),
                      numpy_extended_sum2=[NDArray[int,:,:,:,:]])

    def test_extended_sum3(self):
        self.run_test("def numpy_extended_sum3(a): import numpy ; return numpy.sum(a[1:-1])",
                      numpy.arange(120).reshape((3,5,4,2)),
                      numpy_extended_sum3=[NDArray[int,:,:,:,:]])

    def test_extended_sum4(self):
        self.run_test("def numpy_extended_sum4(a): import numpy ; return numpy.sum(a[1:-1,0])",
                      numpy.arange(120).reshape((3,5,4,2)),
                      numpy_extended_sum4=[NDArray[int,:,:,:,:]])

    def test_extended_sum5(self):
        self.run_test("def numpy_extended_sum5(a): import numpy ; return numpy.sum(a)",
                      numpy.arange(120).reshape((3,5,4,2)),
                      numpy_extended_sum5=[NDArray[int,:,:,:,:]])

    def test_out_sum0(self):
        self.run_test("def numpy_out_sum0(a, b): import numpy ; return numpy.sum(a, axis=0, out=b)",
                      numpy.arange(10).reshape((5,2)),
                      numpy.zeros(2, dtype=int),
                      numpy_out_sum0=[NDArray[int,:,:], NDArray[int,:]])

    def test_out_sum1(self):
        self.run_test("def numpy_out_sum1(a, b): import numpy ; return numpy.sum(a, axis=0, out=b)",
                      numpy.arange(10).reshape((5,2)),
                      numpy.ones(2, dtype=int),
                      numpy_out_sum1=[NDArray[int,:,:], NDArray[int,:]])

    def test_out_sum2(self):
        self.run_test("def numpy_out_sum2(a, b): import numpy ; return numpy.sum(a, axis=1, out=b)",
                      numpy.arange(10).reshape((5,2)),
                      numpy.zeros(5, dtype=int),
                      numpy_out_sum2=[NDArray[int,:,:], NDArray[int,:]])


    def test_numpy_shape_as_function(self):
         self.run_test("def numpy_shape_as_function(a): import numpy ; return numpy.shape(a)",
                       numpy.ones(3, numpy.int16),
                       numpy_shape_as_function=[NDArray[numpy.int16,:]])

    def test_numpy_size_as_function(self):
         self.run_test("def numpy_size_as_function(a): import numpy ; return numpy.size(a)",
                       numpy.ones(3, numpy.int16),
                       numpy_size_as_function=[NDArray[numpy.int16,:]])

    def test_numpy_ndim_as_function(self):
         self.run_test("def numpy_ndim_as_function(a): import numpy ; return numpy.ndim(a)",
                       numpy.ones(3, numpy.int16),
                       numpy_ndim_as_function=[NDArray[numpy.int16,:]])

    def test_frexp0(self):
        self.run_test("def np_frexp0(a): import numpy as np ; return np.frexp(a)", 1.5, np_frexp0=[float])

    def test_frexp1(self):
        self.run_test("def np_frexp1(a): import numpy as np ; return np.frexp(a)", numpy.array([1.1,2.2,3.3]), np_frexp1=[NDArray[float,:]])

    def test_frexp2(self):
        self.run_test("def np_frexp2(a): import numpy as np ; return np.frexp(a+a)", numpy.array([1.1,2.2,3.3]), np_frexp2=[NDArray[float,:]])

    def test_ndindex0(self):
        self.run_test("def np_ndindex0(): import numpy as np ; return [x for x in np.ndindex(5,6)]",
                      np_ndindex0=[])

    def test_ndindex1(self):
        self.run_test("def np_ndindex1(a): import numpy as np ; return [x for x in np.ndindex(a)]", 3, np_ndindex1=[int])

    def test_ndindex2(self):
        self.run_test("def np_ndindex2(n): import numpy as np ; return [x for x in np.ndindex((n,n))]", 3, np_ndindex2=[int])

    def test_ndenumerate0(self):
        self.run_test("def np_ndenumerate0(a): import numpy as np ; return [x for x in np.ndenumerate(a)]", numpy.array([[1, 2], [3, 4]]), np_ndenumerate0=[NDArray[int,:,:]])

    def test_ndenumerate1(self):
        self.run_test("def np_ndenumerate1(a): import numpy as np ; return [x for x in np.ndenumerate(a)]", numpy.array([1, 2, 3, 4]), np_ndenumerate1=[NDArray[int,:]])

    def test_nansum0(self):
        self.run_test("def np_nansum0(a): import numpy as np  ; return np.nansum(a)" , numpy.array([[1, 2], [3, numpy.nan]]), np_nansum0=[NDArray[float,:,:]])

    def test_nansum1(self):
        self.run_test("def np_nansum1(a): import numpy as np ; return np.nansum(a)" , numpy.array([[1, 2], [numpy.NINF, numpy.nan]]), np_nansum1=[NDArray[float,:,:]])

    def test_nansum2(self):
        self.run_test("def np_nansum2(a): import numpy as np ; return np.nansum(a)", [1., numpy.nan], np_nansum2=[List[float]])

    def test_nanmin0(self):
        self.run_test("def np_nanmin0(a): import numpy as np ; return np.nanmin(a)" , numpy.array([[1, 2], [3, numpy.nan]]), np_nanmin0=[NDArray[float,:,:]])

    def test_nanmin1(self):
        self.run_test("def np_nanmin1(a): import numpy as np ; return np.nanmin(a)" , numpy.array([[1, 2], [numpy.NINF, numpy.nan]]), np_nanmin1=[NDArray[float,:,:]])

    def test_nanmin2(self):        
         self.run_test("def np_nanmin2(a): import numpy as np ; return np.nanmin(a)" , numpy.array([[numpy.nan, numpy.nan], [numpy.nan, numpy.nan]]), np_nanmin2=[NDArray[float,:,:]])

    def test_nanmax0(self):
        self.run_test("def np_nanmax0(a): import numpy as np ; return np.nanmax(a)" , numpy.array([[1, 2], [3, numpy.nan]]),  np_nanmax0=[NDArray[float,:,:]])

    def test_nanmax1(self):
        self.run_test("def np_nanmax1(a): import numpy as np ; return np.nanmax(a)" , numpy.array([[1, 2], [numpy.inf, numpy.nan]]) , np_nanmax1=[NDArray[float,:,:]])

    def test_nanmax2(self):        
         self.run_test("def np_nanmax2(a): import numpy as np ; return np.nanmax(a)" , numpy.array([[numpy.nan, numpy.nan], [numpy.nan, numpy.nan]]), np_nanmax2=[NDArray[float,:,:]])

    def test_np_residual(self):
        self.run_test("""import numpy as np
def np_residual():
    nx, ny, nz= 75, 75, 100
    hx, hy = 1./(nx-1), 1./(ny-1)

    P_left, P_right = 0, 0
    P_top, P_bottom = 1, 0
    P = np.ones((nx, ny, nz), np.float64)
    d2x = np.zeros_like(P)
    d2y = np.zeros_like(P)

    d2x[1:-1] = (P[2:]   - 2*P[1:-1] + P[:-2]) / hx/hx
    d2x[0]    = (P[1]    - 2*P[0]    + P_left)/hx/hx
    d2x[-1]   = (P_right - 2*P[-1]   + P[-2])/hx/hx

    d2y[:,1:-1] = (P[:,2:] - 2*P[:,1:-1] + P[:,:-2])/hy/hy
    d2y[:,0]    = (P[:,1]  - 2*P[:,0]    + P_bottom)/hy/hy
    d2y[:,-1]   = (P_top   - 2*P[:,-1]   + P[:,-2])/hy/hy

    return d2x + d2y + 5*np.cosh(P).mean()**2
""", np_residual=[])

    def test_np_func2(self):
        self.run_test("""import numpy as np
def np_func2(x):
    f = [x[0] * np.cos(x[1]) - 4,
         x[1]*x[0] - x[1] - 5]
    df = np.array([[np.cos(x[1]), -x[0] * np.sin(x[1])],
                   [x[1], x[0] - 1]])
    return f, df
""", [1.0, 2.0, 3.0], np_func2=[List[float]])

    def test_np_peval(self):
        self.run_test("""import numpy
def np_peval(x, p):
    return p[0]*numpy.sin(2*numpy.pi*p[1]*x+p[2])
""", 12., [1.0, 2.0, 3.0], np_peval=[float, List[float]])

    def test_np_residuals(self):
        self.run_test("""import numpy
def np_residuals():
    x = numpy.arange(0,6e-2,6e-2/30)
    A,k,theta = 10, 1.0/3e-2, numpy.pi/6
    return A*numpy.sin(2*numpy.pi*k*x+theta)
""", np_residuals=[])

    def test_np_func_deriv(self):
        self.run_test("""import numpy
def np_func_deriv(x, sign=1.0):
    dfdx0 = sign*(-2*x[0] + 2*x[1] + 2)
    dfdx1 = sign*(2*x[0] - 4*x[1])
    return numpy.array([ dfdx0, dfdx1 ])
""", [-1.0, 1.0], -1.0, np_func_deriv=[List[float], float])

    def test_np_func(self):
        self.run_test("""import numpy
def np_func(x, sign=1.0):
    return sign*(2*x[0]*x[1] + 2*x[0] - x[0]**2 - 2*x[1]**2)
""", [-1.0, 1.0], -1.0, np_func=[List[float], float])

    def test_rosen_hess_p(self):
        self.run_test("""import numpy
def np_rosen_hess_p(x, p):
    x = numpy.asarray(x)
    Hp = numpy.zeros_like(x)
    Hp[0] = (1200*x[0]**2 - 400*x[1] + 2)*p[0] - 400*x[0]*p[1]
    Hp[1:-1] = -400*x[:-2]*p[:-2]+(202+1200*x[1:-1]**2-400*x[2:])*p[1:-1] \
               -400*x[1:-1]*p[2:]
    Hp[-1] = -400*x[-2]*p[-2] + 200*p[-1]
    return Hp
""",
                      numpy.array([1.3, 0.7, 0.8, 1.9, 1.2]),
                      numpy.array([2.3, 1.7, 1.8, 2.9, 2.2]),
                      np_rosen_hess_p=[NDArray[float,:], NDArray[float,:]])

    def test_rosen_hess(self):
        self.run_test("""import numpy
def np_rosen_hess(x):
    x = numpy.asarray(x)
    H = numpy.diag(-400*x[:-1],1) - numpy.diag(400*x[:-1],-1)
    diagonal = numpy.zeros_like(x)
    diagonal[0] = 1200*x[0]**2-400*x[1]+2
    diagonal[-1] = 200
    diagonal[1:-1] = 202 + 1200*x[1:-1]**2 - 400*x[2:]
    H = H + numpy.diag(diagonal)
    return H
""",
                      numpy.array([1.3, 0.7, 0.8, 1.9, 1.2]),
                      np_rosen_hess=[NDArray[float,:]])

    def test_rosen_der(self):
        self.run_test("""import numpy
def np_rosen_der(x):
    xm = x[1:-1]
    xm_m1 = x[:-2]
    xm_p1 = x[2:]
    der = numpy.zeros_like(x)
    der[1:-1] = 200*(xm-xm_m1**2) - 400*(xm_p1 - xm**2)*xm - 2*(1-xm)
    der[0] = -400*x[0]*(x[1]-x[0]**2) - 2*(1-x[0])
    der[-1] = 200*(x[-1]-x[-2]**2)
    return der
""",
                      numpy.array([1.3, 0.7, 0.8, 1.9, 1.2]),
                      np_rosen_der=[NDArray[float,:]])

    def test_rosen(self):
        self.run_test("import numpy\ndef np_rosen(x): return sum(100.0*(x[1:]-x[:-1]**2.0)**2.0 + (1-x[:-1])**2.0)",
                      numpy.array([1.3, 0.7, 0.8, 1.9, 1.2]),
                      np_rosen=[NDArray[float,:]])

    def test_nanargmax0(self):
        self.run_test("def np_nanargmax0(a): from numpy import nanargmax; return nanargmax(a)", numpy.array([[numpy.nan, 4], [2, 3]]),  np_nanargmax0=[NDArray[float,:,:]])

    def test_nanargmin0(self):
        self.run_test("def np_nanargmin0(a): from numpy import nanargmin ; return nanargmin(a)", numpy.array([[numpy.nan, 4], [2, 3]]), np_nanargmin0=[NDArray[float,:,:]])

    def test_nan_to_num0(self):
        self.run_test("def np_nan_to_num0(a): import numpy as np ; return np.nan_to_num(a)", numpy.array([numpy.inf, -numpy.inf, numpy.nan, -128, 128]), np_nan_to_num0=[NDArray[float,:]])

    def test_median0(self):
        self.run_test("def np_median0(a): from numpy import median ; return median(a)", numpy.array([[1, 2], [3, 4]]), np_median0=[NDArray[int,:,:]])

    def test_median1(self):
        self.run_test("def np_median1(a): from numpy import median ; return median(a)", numpy.array([1, 2, 3, 4,5]), np_median1=[NDArray[int,:]])

    def test_median2(self):
        self.run_test("def np_median2(a): from numpy import median ; return median(a, None)", numpy.array([1, 2, 3, 4,5]), np_median2=[NDArray[int,:]])

    def test_median3(self):
        self.run_test("def np_median3(a): from numpy import median ; return median(a, 0)", numpy.array([[1, 2, 3], [4,5,6]]), np_median3=[NDArray[int,:,:]])

    def test_median4(self):
        self.run_test("def np_median4(a): from numpy import median ; return median(a, 1)", numpy.array([[1, 2, 3], [4,5,6]]), np_median4=[NDArray[int,:,:]])

    def test_median5(self):
        self.run_test("def np_median5(a): from numpy import median ; return median(a, -1)", numpy.array([[[1], [2], [3]], [[4],[5],[6]]]), np_median5=[NDArray[int,:,:,:]])

    def test_median6(self):
        self.run_test("def np_median6(l): from numpy import median ; return l + median(l)", numpy.array([3, 1]), np_median6=[NDArray[int, :]])

    def test_mean0(self):
        self.run_test("def np_mean0(a): from numpy import mean ; return mean(a)", numpy.array([[1, 2], [3, 4]]), np_mean0=[NDArray[int,:,:]])

    def test_mean1(self):
        self.run_test("def np_mean1(a): from numpy import mean ; return mean(a, 1)", numpy.array([[1, 2], [3, 4.]]), np_mean1=[NDArray[float,:,:]])

    def test_mean2(self):
        self.run_test("def np_mean2(a): from numpy import mean ; return mean(a)", numpy.array([[[1, 2], [3, 4.]]]), np_mean2=[NDArray[float,:,:,:]])

    def test_mean3(self):
        self.run_test("def np_mean3(a): from numpy import mean ; return mean(a, 0)", numpy.array([[[1, 2], [3, 4.]]]), np_mean3=[NDArray[float,:,:,:]])

    def test_mean4(self):
        self.run_test("def np_mean4(a): from numpy import mean ; return mean(a, 1)", numpy.array([[[1, 2], [3, 4.]]]), np_mean4=[NDArray[float,:,:,:]])

    def test_mean5(self):
        self.run_test("def np_mean5(a): from numpy import mean ; return mean(a, 2)", numpy.array([[[1, 2], [3, 4.]]]), np_mean5=[NDArray[float,:,:,:]])

    def test_mean6(self):
        self.run_test("def np_mean6(a): from numpy import mean ; from numpy import float64; return mean(a, 2, float64, None, False)", numpy.array([[[1, 2], [3, 4.]]]), np_mean6=[NDArray[float,:,:,:]])

    def test_mean7(self):
        self.run_test("def np_mean7(a): from numpy import mean ; return mean(a, 2, out=None, keepdims=False)", numpy.array([[[1, 2], [3, 4.]]]), np_mean7=[NDArray[float,:,:,:]])

    def test_mean8(self):
        self.run_test("def np_mean8(a): from numpy import mean ; return mean(a, 2, keepdims=False)", numpy.array([[[1, 2], [3, 4.]]]), np_mean8=[NDArray[float,:,:,:]])

    def test_mean9(self):
        self.run_test("def np_mean9(a): from numpy import mean ; return mean(a, keepdims=False)", numpy.array([[[1, 2], [3, 4.]]]), np_mean9=[NDArray[float,:,:,:]])

    def test_mean10(self):
        self.run_test("def np_mean10(a): from numpy import mean ; return mean(a, 2, dtype=int, keepdims=True)", numpy.array([[[0, 2], [3, 4.]]]), np_mean10=[NDArray[float,:,:,:]])

    def test_mean11(self):
        self.run_test("def np_mean11(a): from numpy import mean ; return mean(a, 2, keepdims=1)", numpy.array([[[1, 2], [3, 4.]]]), np_mean11=[NDArray[float,:,:,:]])

    def test_mean12(self):
        self.run_test("def np_mean12(a): from numpy import mean ; return mean(a, keepdims=True)", numpy.array([[[1, 2], [3, 4.]]]), np_mean12=[NDArray[float,:,:,:]])

    def test_mean13(self):
        self.run_test("def np_mean13(a): from numpy import mean ; return mean(a, keepdims=True, dtype=int)", numpy.array([[[0, 2], [3, 4.]]]), np_mean13=[NDArray[float,:,:,:]])

    def test_var0(self):
        self.run_test("def np_var0(a): return a.var()", numpy.array([[1, 2], [3, 4]], dtype=float), np_var0=[NDArray[float,:,:]])

    def test_var1(self):
        self.run_test("def np_var1(a): from numpy import var ; return var(a, 1)", numpy.array([[1, 2], [3, 4.]]), np_var1=[NDArray[float,:,:]])

    def test_var2(self):
        self.run_test("def np_var2(a): from numpy import var ; return var(a)", numpy.array([[[1, 2], [3, 4.]]]), np_var2=[NDArray[float,:,:,:]])

    def test_var3(self):
        self.run_test("def np_var3(a): from numpy import var ; return var(a, 0)", numpy.array([[[1, 2], [3, 4.]]]), np_var3=[NDArray[float,:,:,:]])

    def test_var4(self):
        self.run_test("def np_var4(a): from numpy import var ; return var(a, 1)", numpy.array([[[1, 2], [3, 4.]]]), np_var4=[NDArray[float,:,:,:]])

    def test_var5(self):
        self.run_test("def np_var5(a): from numpy import var ; return var(a, 2)", numpy.array([[[1, 2], [3, 4.]]]), np_var5=[NDArray[float,:,:,:]])

    def test_var6(self):
        self.run_test("def np_var6(a): from numpy import var ; return var(1j * a)", numpy.array([[[1, 2], [3, 4.]]]), np_var6=[NDArray[float,:,:,:]])

    def test_var7(self):
        self.run_test("def np_var7(a): from numpy import var ; return var(1j * a, 2)", numpy.array([[[1, 2], [3, 4.]]]), np_var7=[NDArray[float,:,:,:]])

    def test_var8(self):
        self.run_test("def np_var8(a): from numpy import var ; return var(1j * a, 2)", numpy.array([[[1, 2], [3, 4]]]), np_var8=[NDArray[int,:,:,:]])

    def test_var9(self):
        self.run_test("def np_var9(a): from numpy import var ; return var(1j * a)", numpy.array([[[1, 2], [3, 4]]]), np_var9=[NDArray[int,:,:,:]])

    def test_std0(self):
        self.run_test("def np_std0(a): from numpy import std ; return std(a)", numpy.array([[[1, 2], [3, 4]]]), np_std0=[NDArray[int, :, :, :]])

    def test_std1(self):
        self.run_test("def np_std1(a): from numpy import std ; return std(a, 0)", numpy.array([[[1, 2], [3, 4]]]), np_std1=[NDArray[int, :, :, :]])

    def test_std2(self):
        self.run_test("def np_std2(a): from numpy import std ; return std(a, 1)", numpy.array([[[1, 2], [3, 4]]]), np_std2=[NDArray[int, :, :, :]])

    def test_std3(self):
        self.run_test("def np_std3(a): from numpy import std ; return std(1j*a, 1)", numpy.array([[[1, 2], [3, 4]]]), np_std3=[NDArray[int, :, :, :]])

    def test_logspace0(self):
        self.run_test("def np_logspace0(start, stop): from numpy import logspace ; start, stop = 3., 4. ; return logspace(start, stop, 4)", 3., 4., np_logspace0=[float, float])

    def test_logspace1(self):
        self.run_test("def np_logspace1(start, stop): from numpy import logspace ; return logspace(start, stop, 4, False)", 3., 4., np_logspace1=[float, float])

    def test_logspace2(self):
        self.run_test("def np_logspace2(start, stop): from numpy import logspace ; return logspace(start, stop, 4, True, 2.0)", 3., 4., np_logspace2=[float, float])

    def test_lexsort0(self):
        self.run_test("def np_lexsort0(surnames): from numpy import lexsort ; first_names = ('Heinrich', 'Galileo', 'Gustav') ; return lexsort((first_names, surnames))", ('Hertz',    'Galilei', 'Hertz'), np_lexsort0=[Tuple[str, str, str]])

    def test_lexsort1(self):
        self.run_test("def np_lexsort1(a): from numpy import lexsort ; b = [1,5,1,4,3,4,4] ; return lexsort((a,b))", [9,4,0,4,0,2,1], np_lexsort1=[List[int]])

    def test_lexsort2(self):
        self.run_test("def np_lexsort2(a): from numpy import lexsort ; return lexsort((a+1,a-1))", numpy.array([1,5,1,4,3,4,4]), np_lexsort2=[NDArray[int,:]])

    def test_issctype0(self):
        self.run_test("def np_issctype0(): from numpy import issctype, int32 ; a = int32 ; return issctype(a)", np_issctype0=[])

    def test_issctype1(self):
        self.run_test("def np_issctype1(): from numpy import issctype ; a = list ; return issctype(a)", np_issctype1=[])

    def test_issctype2(self):
        self.run_test("def np_issctype2(a): from numpy import issctype ; return issctype(a)", 3.1, np_issctype2=[float])

    def test_isscalar0(self):
        self.run_test("def np_isscalar0(a): from numpy import isscalar ; return isscalar(a)", 3.1, np_isscalar0=[float])

    def test_isscalar1(self):
        self.run_test("def np_isscalar1(a): from numpy import isscalar ; return isscalar(a)", [3.1], np_isscalar1=[List[float]])

    def test_isscalar2(self):
        self.run_test("def np_isscalar2(a): from numpy import isscalar ; return isscalar(a)", '3.1', np_isscalar2=[str])

    def test_isrealobj0(self):
        self.run_test("def np_isrealobj0(a): from numpy import isrealobj ; return isrealobj(a)", numpy.array([1,2,3.]), np_isrealobj0=[NDArray[float,:]])

    def test_isrealobj1(self):
        self.run_test("def np_isrealobj1(a): from numpy import isrealobj ; return isrealobj(a)", numpy.array([1,2,3.,4 + 1j]).reshape((2,2)), np_isrealobj1=[NDArray[complex,:,:]])

    def test_isreal0(self):
        self.run_test("def np_isreal0(a): from numpy import isreal ; return isreal(a)", numpy.array([1,2,3.]), np_isreal0=[NDArray[float,:]])

    def test_isreal1(self):
        self.run_test("def np_isreal1(a): from numpy import isreal ; return isreal(a)", numpy.array([1,2,3.,4 + 1j]).reshape((2,2)), np_isreal1=[NDArray[complex,:,:]])

    def test_iscomplex0(self):
        self.run_test("def np_iscomplex0(a): from numpy import iscomplex ; return iscomplex(a)", numpy.array([1, 2, 3.]), np_iscomplex0=[NDArray[float,:]])

    def test_iscomplex1(self):
        self.run_test("def np_iscomplex1(a): from numpy import iscomplex ; return iscomplex(a)", numpy.array([1,2,3.,4 + 1j]).reshape((2,2)), np_iscomplex1=[NDArray[complex,:,:]])

    def test_intersect1d0(self):
        self.run_test("def np_intersect1d0(a): from numpy import intersect1d ; b = [3, 1, 2, 1] ; return intersect1d(a,b)", [1, 3, 4, 3], np_intersect1d0=[List[int]])

    def test_insert0(self):
        self.run_test("def np_insert0(a): from numpy import insert ; return insert(a, 1, 5)", numpy.array([[1, 1], [2, 2], [3, 3]]), np_insert0=[NDArray[int,:,:]])

    def test_insert1(self):
        self.run_test("def np_insert1(a): from numpy import insert ; return insert(a, [1,2], [5,6])", numpy.array([[1, 1], [2, 2], [3, 3]]), np_insert1=[NDArray[int,:,:]])

    def test_insert2(self):
        self.run_test("def np_insert2(a): from numpy import insert ; return insert(a, [1,1], [5.2,6])", numpy.array([[1, 1], [2, 2], [3, 3]]), np_insert2=[NDArray[int,:,:]])

    def test_inner0(self):
        self.run_test("def np_inner0(x): from numpy import inner ; y = 3 ; return inner(x,y)", 2, np_inner0=[int])

    def test_inner1(self):
        self.run_test("def np_inner1(x): from numpy import inner ; y = [2, 3] ; return inner(x,y)", [2, 3], np_inner1=[List[int]])

    def test_indices0(self):
        self.run_test("def np_indices0(s): from numpy import indices ; return indices(s)", (2, 3), np_indices0=[Tuple[int, int]])

    def test_identity0(self):
        self.run_test("def np_identity0(a): from numpy import identity ; return identity(a)", 3, np_identity0=[int])

    def test_identity1(self):
        self.run_test("def np_identity1(a): from numpy import identity ;return identity(a)", 4, np_identity1=[int])

    def test_tofile0(self):
        temp_name = tempfile.mkstemp()[1]
        x = numpy.random.randint(0,2**8,1000).astype(numpy.uint8)
        try:
            self.run_test("def np_tofile0(x,file): import numpy ; x.tofile(file); return numpy.fromfile(file)", x, temp_name, np_tofile0=[NDArray[numpy.uint8,:], str])
        finally:
            os.remove(temp_name)

    def test_tofile1(self):
        temp_name = tempfile.mkstemp()[1]
        x = numpy.random.randint(0,2**16,1000).astype(numpy.uint16)
        try:
            self.run_test("def np_tofile1(x,file): import numpy ; x.tofile(file); return numpy.fromfile(file)", x, temp_name, np_tofile1=[NDArray[numpy.uint16,:], str])
        finally:
            os.remove(temp_name)

    def test_tofile2(self):
        temp_name = tempfile.mkstemp()[1]
        x = numpy.random.randint(0,2**31,1000).astype(numpy.uint32)
        try:
            self.run_test("def np_tofile2(x,file): import numpy ; x.tofile(file); return numpy.fromfile(file)", x, temp_name, np_tofile2=[NDArray[numpy.uint32,:], str])
        finally:
            os.remove(temp_name)

    def test_tofile3(self):
        temp_name = tempfile.mkstemp()[1]
        x = numpy.random.random(1000).astype(numpy.float32)
        try:
            self.run_test("def np_tofile3(x,file): import numpy ; x.tofile(file); return numpy.fromfile(file)", x, temp_name, np_tofile3=[NDArray[numpy.float32,:], str])
        finally:
            os.remove(temp_name)

    def test_tofile4(self):
        temp_name = tempfile.mkstemp()[1]
        x = numpy.random.random(1000).astype(numpy.float64)
        try:
            self.run_test("def np_tofile4(x,file): import numpy ; x.tofile(file); return numpy.fromfile(file)", x, temp_name, np_tofile4=[NDArray[numpy.float64,:], str])
        finally:
            os.remove(temp_name)

    def test_fromfile0(self):
        temp_name = tempfile.mkstemp()[1]
        x = numpy.random.randint(0,2**8,1000).astype(numpy.uint8)
        x.tofile(temp_name)
        try:
            self.run_test("def np_fromfile0(file): from numpy import fromfile, uint8 ; return fromfile(file, uint8)", temp_name, np_fromfile0=[str])
        finally:
            os.remove(temp_name)

    def test_fromfile1(self):
        temp_name = tempfile.mkstemp()[1]
        x = numpy.random.randint(0,2**16,1000).astype(numpy.uint16)
        x.tofile(temp_name)
        try:
            self.run_test("def np_fromfile1(file): from numpy import fromfile, uint16 ; return fromfile(file, uint16)", temp_name, np_fromfile1=[str])
        finally:
            os.remove(temp_name)

    def test_fromfile2(self):
        temp_name = tempfile.mkstemp()[1]
        x = numpy.random.randint(0,2**31,1000).astype(numpy.uint32)
        x.tofile(temp_name)
        try:
            self.run_test("def np_fromfile2(file): from numpy import fromfile, uint32 ; return fromfile(file, uint32)", temp_name, np_fromfile2=[str])
        finally:
            os.remove(temp_name)

    def test_fromfile3(self):
        temp_name = tempfile.mkstemp()[1]
        x = numpy.random.random(1000).astype(numpy.float32)
        x.tofile(temp_name)
        try:
            self.run_test("def np_fromfile3(file): from numpy import fromfile, float32 ; return fromfile(file, float32)", temp_name, np_fromfile3=[str])
        finally:
            os.remove(temp_name)

    def test_fromfile4(self):
        temp_name = tempfile.mkstemp()[1]
        x = numpy.random.random(1000).astype(numpy.float64)
        x.tofile(temp_name)
        try:
            self.run_test("def np_fromfile4(file): from numpy import fromfile, float64 ; return fromfile(file, float64)", temp_name, np_fromfile4=[str])
        finally:
            os.remove(temp_name)

    def test_fromfile5(self):
        temp_name = tempfile.mkstemp()[1]
        x = numpy.random.random(1000).astype(numpy.float64)
        x.tofile(temp_name)
        try:
            self.run_test("def np_fromfile5(file): from numpy import fromfile, float64 ; return fromfile(file, float64, 100)", temp_name, np_fromfile5=[str])
        finally:
            os.remove(temp_name)

    def test_fromstring0(self):
        self.run_test("def np_fromstring0(a): from numpy import fromstring, uint8 ; return fromstring(a, uint8)", '\x01\x02', np_fromstring0=[str])

    def test_fromstring1(self):
        self.run_test("def np_fromstring1(a): from numpy import fromstring, uint8 ; a = '\x01\x02\x03\x04' ; return fromstring(a, uint8,3)", '\x01\x02\x03\x04', np_fromstring1=[str])

    def test_fromstring2(self):
        self.run_test("def np_fromstring2(a): from numpy import fromstring, uint32 ; return fromstring(a, uint32,-1, ' ')", '1 2 3 4', np_fromstring2=[str])

    def test_fromstring3(self):
        self.run_test("def np_fromstring3(a): from numpy import fromstring, uint32 ; return fromstring(a, uint32,2, ',')", '1,2, 3, 4', np_fromstring3=[str])

    def test_outer0(self):
        self.run_test("def np_outer0(x): from numpy import outer ; return outer(x, x+2)", numpy.arange(6).reshape(2,3), np_outer0=[NDArray[int,:,:]])

    def test_outer1(self):
        self.run_test("def np_outer1(x): from numpy import outer; return outer(x, range(6))", numpy.arange(6).reshape((2,3)), np_outer1=[NDArray[int,:,:]])

    def test_place0(self):
        self.run_test("def np_place0(x): from numpy import place, ravel ; place(x, x>1, ravel(x**2)); return x", numpy.arange(6).reshape((2,3)), np_place0=[NDArray[int,:,:]])

    def test_place1(self):
        self.run_test("def np_place1(x): from numpy import place ; place(x, x>1, [57, 58]); return x", numpy.arange(6).reshape((2,3)), np_place1=[NDArray[int,:,:]])

    def test_product(self):
        self.run_test("def np_product(x):\n from numpy import product\n return product(x)", numpy.arange(1, 10), np_product=[NDArray[int,:]])

    def test_ptp0(self):
        self.run_test("def np_ptp0(x): return x.ptp()", numpy.arange(4).reshape((2,2)), np_ptp0=[NDArray[int,:,:]])

    def test_ptp1(self):
        self.run_test("def np_ptp1(x): from numpy import ptp ; return ptp(x,0)", numpy.arange(4).reshape((2,2)), np_ptp1=[NDArray[int,:,:]])

    def test_ptp2(self):
        self.run_test("def np_ptp2(x): from numpy import ptp ; return ptp(x,1)", numpy.arange(4).reshape((2,2)), np_ptp2=[NDArray[int,:,:]])

    def test_put0(self):
        self.run_test("def np_put0(x): x.put([0,2], [-44, -55]); return x", numpy.arange(5), np_put0=[NDArray[int,:]])

    def test_put1(self):
        self.run_test("def np_put1(x): from numpy import put ; put(x, [0,2,3], [57, 58]); return x", numpy.arange(6).reshape((2, 3)), np_put1=[NDArray[int,:,:]])

    def test_put2(self):
        self.run_test("def np_put2(x): from numpy import put ; put(x, 2, 57); return x", numpy.arange(6).reshape((2,3)), np_put2=[NDArray[int,:,:]])

    def test_putmask0(self):
        self.run_test("def np_putmask0(x): from numpy import putmask ; putmask(x, x>1, x**2); return x", numpy.arange(6).reshape((2,3)), np_putmask0=[NDArray[int,:,:]])

    def test_putmask1(self):
        self.run_test("def np_putmask1(x): from numpy import putmask; putmask(x, x>1, [57, 58]); return x", numpy.arange(6).reshape((2,3)), np_putmask1=[NDArray[int,:,:]])

    def test_ravel0(self):
        self.run_test("def np_ravel0(x): from numpy import ravel ; return ravel(x)", numpy.arange(6).reshape((2,3)), np_ravel0=[NDArray[int,:,:]])

    def test_ravel1(self):
        self.run_test("def np_ravel1(x): return x.ravel()", numpy.arange(6).reshape((2,3)), np_ravel1=[NDArray[int,:,:]])

    def test_ravel2(self):
        self.run_test("def np_ravel2(x): y = x.ravel(); y[3] = 10; return x", numpy.arange(6).reshape((2,3)), np_ravel2=[NDArray[int,:,:]])

    def test_ravel3(self):
        code = '''
import numpy as np

def advance_positions(positions, velocities, time_step):
    positions += time_step * velocities

def advance_positions_ravel(positions, velocities, time_step):
    positions_flat = positions.ravel()
    velocities_flat = velocities.ravel()
    positions_flat += time_step * velocities_flat

def np_ravel3(v):
    time_step = v

    positions = np.zeros((2, 2))
    velocities = np.ones_like(positions)

    advance_positions(positions, velocities, time_step)
    r0 = positions[0, 0]

    positions = np.zeros((2, 2))
    advance_positions_ravel(positions, velocities, time_step)
    r1 = positions[0, 0]
    return r0, r1
        '''
        self.run_test(code, 1, np_ravel3=[int])

    def test_repeat0(self):
        self.run_test("def np_repeat0(x): from numpy import repeat; return repeat(x, 3)", numpy.arange(3), np_repeat0=[NDArray[int,:]])

    def test_repeat1(self):
        self.run_test("def np_repeat1(x): return x.repeat(3)", numpy.arange(6).reshape(2,3), np_repeat1=[NDArray[int,:,:]])

    def test_repeat2(self):
        self.run_test("def np_repeat2(x): from numpy import repeat; return repeat(x, 4, axis=0)", numpy.arange(6).reshape(2,3), np_repeat2=[NDArray[int,:,:]])

    def test_repeat3(self):
        self.run_test("def np_repeat3(x): from numpy import repeat; return repeat(x, 4, axis=1)", numpy.arange(6).reshape(2,3), np_repeat3=[NDArray[int,:,:]])

    def test_resize4(self):
        self.run_test("def np_resize4(x): from numpy import resize ; return resize(x, (6,7))", numpy.arange(24).reshape((2,3,4)), np_resize4=[NDArray[int, :, :, :]])

    def test_resize3(self):
        self.run_test("def np_resize3(x): from numpy import resize; return resize(x, (6,6))", numpy.arange(24).reshape((2,3,4)), np_resize3=[NDArray[int, :, :, :]])

    def test_resize2(self):
        self.run_test("def np_resize2(x): from numpy import resize; return resize(x, (3,3))", numpy.arange(24).reshape((2,3,4)), np_resize2=[NDArray[int, :, :, :]])

    def test_resize1(self):
        self.run_test("def np_resize1(x): from numpy import resize; return resize(x, 32)", numpy.arange(24), np_resize1=[NDArray[int,:]])

    def test_resize0(self):
        self.run_test("def np_resize0(x): from numpy import resize; return resize(x, 12)", numpy.arange(24), np_resize0=[NDArray[int,:]])

    def test_rollaxis3(self):
        self.run_test("def np_rollaxis3(x): from numpy import rollaxis; return rollaxis(x, 0, 3)", numpy.arange(24).reshape((2,3,4)), np_rollaxis3=[NDArray[int, :, :, :]])

    def test_rollaxis2(self):
        self.run_test("def np_rollaxis2(x): from numpy import rollaxis; return rollaxis(x, 2)", numpy.arange(24).reshape((2,3,4)), np_rollaxis2=[NDArray[int, :, :, :]])

    def test_rollaxis1(self):
        self.run_test("def np_rollaxis1(x): from numpy import rollaxis; return rollaxis(x, 1, 2)", numpy.arange(24).reshape(2,3,4), np_rollaxis1=[NDArray[int, :, :, :]])

    def test_rollaxis0(self):
        self.run_test("def np_rollaxis0(x): from numpy import rollaxis; return rollaxis(x, 1)", numpy.arange(24).reshape(2,3,4), np_rollaxis0=[NDArray[int, :, :, :]])

    def test_roll6(self):
        self.run_test("def np_roll6(x): from numpy import roll; return roll(x[:,:,:-1], -1, 2)", numpy.arange(24).reshape(2,3,4), np_roll6=[NDArray[int, :, :, :]])

    def test_roll5(self):
        self.run_test("def np_roll5(x): from numpy import roll; return roll(x, -1, 2)", numpy.arange(24).reshape(2,3,4), np_roll5=[NDArray[int, :, :, :]])

    def test_roll4(self):
        self.run_test("def np_roll4(x): from numpy import roll; return roll(x, 1, 1)", numpy.arange(24).reshape(2,3,4), np_roll4=[NDArray[int, :, :, :]])

    def test_roll3(self):
        self.run_test("def np_roll3(x): from numpy import roll; return roll(x, -1, 0)", numpy.arange(24).reshape(2,3,4), np_roll3=[NDArray[int, :, :, :]])

    def test_roll2(self):
        self.run_test("def np_roll2(x): from numpy import roll; return roll(x, -1)", numpy.arange(24).reshape(2,3,4), np_roll2=[NDArray[int, :, :, :]])

    def test_roll1(self):
        self.run_test("def np_roll1(x): from numpy import roll; return roll(x, 10)", numpy.arange(24).reshape(2,3,4), np_roll1=[NDArray[int, :, :, :]])

    def test_roll0(self):
        self.run_test("def np_roll0(x): from numpy import roll; return roll(x, 3)", numpy.arange(24).reshape(2,3,4), np_roll0=[NDArray[int, :, :, :]])

    def test_searchsorted3(self):
        self.run_test("def np_searchsorted3(x): from numpy import searchsorted; return searchsorted(x, [[3,4],[1,87]])", numpy.arange(6), np_searchsorted3=[NDArray[int,:]])

    def test_searchsorted2(self):
        self.run_test("def np_searchsorted2(x): from numpy import searchsorted; return searchsorted(x, [[3,4],[1,87]], 'right')", numpy.arange(6), np_searchsorted2=[NDArray[int,:]])

    def test_searchsorted1(self):
        self.run_test("def np_searchsorted1(x): from numpy import searchsorted; return searchsorted(x, 3)", numpy.arange(6), np_searchsorted1=[NDArray[int,:]])

    def test_searchsorted0(self):
        self.run_test("def np_searchsorted0(x): from numpy import searchsorted; return x.searchsorted(3, 'right')", numpy.arange(6), np_searchsorted0=[NDArray[int,:]])

    def test_rot904(self):
        self.run_test("def np_rot904(x): from numpy import rot90; return rot90(x, 4)", numpy.arange(24).reshape(2,3,4), np_rot904=[NDArray[int, :, :, :]])

    def test_rot903(self):
        self.run_test("def np_rot903(x): from numpy import rot90; return rot90(x, 2)", numpy.arange(24).reshape(2,3,4), np_rot903=[NDArray[int, :, :, :]])

    def test_rot902(self):
        self.run_test("def np_rot902(x): from numpy import rot90; return rot90(x, 3)", numpy.arange(24).reshape(2,3,4), np_rot902=[NDArray[int, :, :, :]])

    def test_rot900(self):
        self.run_test("def np_rot900(x): from numpy import rot90; return rot90(x)", numpy.arange(24).reshape(2,3,4), np_rot900=[NDArray[int, :, :, :]])

    def test_rot901(self):
        self.run_test("def np_rot901(x): from numpy import rot90; return rot90(x)", numpy.arange(4).reshape(2,2), np_rot901=[NDArray[int,:,:]])

    def test_select2(self):
        self.run_test("def np_select2(x): from numpy import select; condlist = [x<3, x>5]; choicelist = [x**3, x**2]; return select(condlist, choicelist)", numpy.arange(10).reshape(2,5), np_select2=[NDArray[int,:,:]])

    def test_select1(self):
        self.run_test("def np_select1(x): from numpy import select; condlist = [x<3, x>5]; choicelist = [x+3, x**2]; return select(condlist, choicelist)", numpy.arange(10), np_select1=[NDArray[int,:]])

    def test_select0(self):
        self.run_test("def np_select0(x): from numpy import select; condlist = [x<3, x>5]; choicelist = [x, x**2]; return select(condlist, choicelist)", numpy.arange(10), np_select0=[NDArray[int,:]])

    def test_sometrue0(self):
        self.run_test("def np_sometrue0(a): from numpy import sometrue ; return sometrue(a)", numpy.array([[True, False], [True, True]]), np_sometrue0=[NDArray[bool,:,:]])

    def test_sometrue1(self):
        self.run_test("def np_sometrue1(a): from numpy import sometrue ; return sometrue(a, 0)", numpy.array([[True, False], [False, False]]), np_sometrue1=[NDArray[bool,:,:]])

    def test_sometrue2(self):
        self.run_test("def np_sometrue2(a): from numpy import sometrue ; return sometrue(a)", [-1, 0, 5], np_sometrue2=[List[int]])

    def test_sort0(self):
        self.run_test("def np_sort0(a): from numpy import sort ; return sort(a)", numpy.array([[1,6],[7,5]]), np_sort0=[NDArray[int,:,:]])

    def test_sort1(self):
        self.run_test("def np_sort1(a): from numpy import sort ; return sort(a)", numpy.array([2, 1, 6, 3, 5]), np_sort1=[NDArray[int,:]])

    def test_sort2(self):
        self.run_test("def np_sort2(a): from numpy import sort ; return sort(a)", numpy.arange(2*3*4, 0, -1).reshape(2,3,4), np_sort2=[NDArray[int, :, :, :]])

    def test_sort3(self):
        self.run_test("def np_sort3(a): from numpy import sort ; return sort(a, 0)", numpy.arange(2*3*4, 0, -1).reshape(2,3,4), np_sort3=[NDArray[int, :, :, :]])

    def test_sort4(self):
        self.run_test("def np_sort4(a): from numpy import sort ; return sort(a, 1, kind='quicksort')", numpy.arange(2*3*4, 0, -1).reshape(2,3,4), np_sort4=[NDArray[int, :, :, :]])

    def test_sort5(self):
        self.run_test("def np_sort5(a): from numpy import sort ; return sort(a, 1, kind='heapsort')", numpy.arange(2*3*5, 0, -1).reshape(2,3,5), np_sort5=[NDArray[int, :, :, :]])

    def test_sort6(self):
        self.run_test("def np_sort6(a): from numpy import sort ; return sort(a, 0, kind='stable')", numpy.arange(2*3*6, 0, -1).reshape(2,3,6), np_sort6=[NDArray[int, :, :, :]])

    def test_sort7(self):
        self.run_test("def np_sort7(a): from numpy import sort ; return sort(a, 2, kind='mergesort')", numpy.arange(2*3*7, 0, -1).reshape(2,3,7), np_sort7=[NDArray[int, :, :, :]])

    def test_sort8(self):
        self.run_test("def np_sort8(a): from numpy import sort ; return sort(a, None)", numpy.arange(2*3*7, 0, -1).reshape(2,3,7), np_sort8=[NDArray[int, :, :, :]])

    def test_sort9(self):
        self.run_test("def np_sort9(a): from numpy import sort ; return sort(2 * a, None)", numpy.arange(2*3*7, 0, -1).reshape(2,3,7), np_sort9=[NDArray[int, :, :, :]])

    def test_sort10(self):
        self.run_test("def np_sort10(a): from numpy import sort ; return sort(3*a, 0)", numpy.arange(2*3*4, 0, -1).reshape(2,3,4), np_sort10=[NDArray[int, :, :, :]])

    def test_sort11(self):
        self.run_test("def np_sort11(a): a.sort(kind='heapsort'); return a", numpy.arange(2*3*4, 0, -1).reshape(2,3,4), np_sort11=[NDArray[int, :, :, :]])

    def test_sort_complex0(self):
        self.run_test("def np_sort_complex0(a): from numpy import sort_complex ; return sort_complex(a)", numpy.array([[1,6],[7,5]]), np_sort_complex0=[NDArray[int,:,:]])

    def test_sort_complex1(self):
        self.run_test("def np_sort_complex1(a): from numpy import sort_complex ; return sort_complex(a)", numpy.array([1 + 2j, 2 - 1j, 3 - 2j, 3 - 3j, 3 + 5j]), np_sort_complex1=[NDArray[complex,:]])

    def test_split0(self):
        self.run_test("def np_split0(a): from numpy import split,array2string ; return list(map(array2string,split(a, 3)))", numpy.arange(12), np_split0=[NDArray[int,:]])

    def test_split1(self):
        self.run_test("def np_split1(a):\n from numpy import split\n try:\n  print(split(a, 5))\n  return False\n except ValueError:\n  return True", numpy.arange(12), np_split1=[NDArray[int,:]])

    def test_split2(self):
        self.run_test("def np_split2(a): from numpy import split, array2string; return list(map(array2string,split(a, [0,1,5])))", numpy.arange(12).reshape(6,2), np_split2=[NDArray[int,:,:]])

    @unittest.skip("Require numpy_fexpr for multidim array")
    def test_take0(self):
        self.run_test("def np_take0(a):\n from numpy import take\n return take(a, [0,1])", numpy.arange(24).reshape(2,3,4), np_take0=[NDArray[int, :, :, :]])

    @unittest.skip("Require numpy_fexpr for multidim array")
    def test_take1(self):
        self.run_test("def np_take1(a):\n from numpy import take\n return take(a, [[0,0,2,2],[1,0,1,2]])", numpy.arange(24).reshape(2,3,4), np_take1=[NDArray[int, :, :, :]])

    @unittest.skip("Require numpy_fexpr with indices")
    def test_take2(self):
        self.run_test("def np_take2(a):\n from numpy import take\n return take(a, [1,0,1,2])", numpy.arange(24), np_take2=[NDArray[int,:]])

    def test_swapaxes_(self):
        self.run_test("def np_swapaxes_(a):\n from numpy import swapaxes\n return swapaxes(a, 1, 2)", numpy.arange(24).reshape(2,3,4), np_swapaxes_=[NDArray[int, :, :, :]])

    def test_tile0(self):
        self.run_test("def np_tile0(a): from numpy import tile ; return tile(a, 3)", numpy.arange(4), np_tile0=[NDArray[int,:]])

    def test_tile1(self):
        self.run_test("def np_tile1(a): from numpy import tile ; return tile(a, (3, 2))", numpy.arange(4), np_tile1=[NDArray[int,:]])

    def test_tile2(self):
        self.run_test("def np_tile2(a): from numpy import tile ; return tile(a, (1, 2))", numpy.arange(12).reshape(3,4), np_tile2=[NDArray[int,:,:]])

    def test_tolist0(self):
        self.run_test("def np_tolist0(a): return a.tolist()", numpy.arange(12), np_tolist0=[NDArray[int,:]])

    def test_tolist1(self):
        self.run_test("def np_tolist1(a): return a.tolist()", numpy.arange(12).reshape(3,4), np_tolist1=[NDArray[int,:,:]])

    def test_tolist2(self):
        self.run_test("def np_tolist2(a): return a.tolist()", numpy.arange(2*3*4*5).reshape(2,3,4,5), np_tolist2=[NDArray[int, :, :, :, :]])

    @unittest.skip("bytes/str confusion")
    def test_tostring0(self):
        self.run_test("def np_tostring0(a): return a.tostring()", numpy.arange(80, 100), np_tostring0=[NDArray[int,:]])

    @unittest.skip("bytes/str confusion")
    def test_tostring1(self):
        self.run_test("def np_tostring1(a): return a.tostring()", numpy.arange(500, 600), np_tostring1=[NDArray[int,:]])

    def test_fromiter0(self):
        self.run_test("def g(): yield 1 ; yield 2\ndef np_fromiter0(): from numpy import fromiter, float32 ; iterable = g() ; return fromiter(iterable, float32)", np_fromiter0=[])

    def test_fromiter1(self):
        self.run_test("def np_fromiter1(): from numpy import fromiter, float32 ; iterable = (x*x for x in range(5)) ; return fromiter(iterable, float32, 5)", np_fromiter1=[])

    def test_fromiter2(self):
        self.run_test("def np_fromiter2(): from numpy import fromiter, float64 ; iterable = (x-x for x in range(5)) ; return fromiter(iterable, count=2, dtype=float64)", np_fromiter2=[])

    def test_fromfunction0(self):
        self.run_test("def np_fromfunction0(s): from numpy import fromfunction ; return fromfunction(lambda i: i == 1, s)", (3,), np_fromfunction0=[Tuple[int]])

    def test_fromfunction1(self):
        self.run_test("def np_fromfunction1(s): from numpy import fromfunction; return fromfunction(lambda i, j: i + j, s)", (3, 3), np_fromfunction1=[Tuple[int, int]])

    def test_flipud0(self):
        self.run_test("def np_flipud0(x): from numpy import flipud ; return flipud(x)", numpy.arange(9).reshape(3,3), np_flipud0=[NDArray[int,:,:]])

    def test_fliplr0(self):
        self.run_test("def np_fliplr0(x): from numpy import fliplr ; return fliplr(x)", numpy.arange(9).reshape(3,3), np_fliplr0=[NDArray[int,:,:]])

    def test_flip3(self):
        self.run_test("def np_flip3(x): from numpy import flip; return flip(x[:,:,:-1], 2)", numpy.arange(24).reshape(2,3,4), np_flip3=[NDArray[int, :, :, :]])

    def test_flip2(self):
        self.run_test("def np_flip2(x): from numpy import flip; return flip(x, 2)", numpy.arange(24).reshape(2,3,4), np_flip2=[NDArray[int, :, :, :]])

    def test_flip1(self):
        self.run_test("def np_flip1(x): from numpy import flip; return flip(x, 1)", numpy.arange(24).reshape(2,3,4), np_flip1=[NDArray[int, :, :, :]])

    def test_flip0(self):
        self.run_test("def np_flip0(x): from numpy import flip; return flip(x, 0)", numpy.arange(24).reshape(2,3,4), np_flip0=[NDArray[int, :, :, :]])

    def test_flatten0(self):
        self.run_test("def np_flatten0(x): return x.flatten()", numpy.array([[1,2], [3,4]]), np_flatten0=[NDArray[int,:,:]])

    def test_flatnonzero0(self):
        self.run_test("def np_flatnonzero0(x): from numpy import flatnonzero ; return flatnonzero(x)", numpy.arange(-2, 3), np_flatnonzero0=[NDArray[int,:]])

    def test_flatnonzero1(self):
        self.run_test("def np_flatnonzero1(x): from numpy import flatnonzero ;  return flatnonzero(x[1:-1])", numpy.arange(-2, 3), np_flatnonzero1=[NDArray[int,:]])

    def test_fix0(self):
        self.run_test("def np_fix0(x): from numpy import fix ; return fix(x)", 3.14, np_fix0=[float])

    def test_fix1(self):
        self.run_test("def np_fix1(x): from numpy import fix ; return fix(x)", 3, np_fix1=[int])

    def test_fix2(self):
        self.run_test("def np_fix2(x): from numpy import fix ; return fix(x)", numpy.array([2.1, 2.9, -2.1, -2.9]), np_fix2=[NDArray[float,:]])

    def test_fix3(self):
        self.run_test("def np_fix3(x): from numpy import fix ; return fix(x)", numpy.array([2.1, 2.9, -2.1, -2.9]), np_fix3=[NDArray[float,:]])

    def test_fix4(self):
        self.run_test("def np_fix4(x): from numpy import fix ; return fix(x+x)", numpy.array([2.1, 2.9, -2.1, -2.9]), np_fix4=[NDArray[float,:]])

    def test_cross1(self):
        self.run_test("def np_cross1(x): from numpy import cross ; return cross(x, [3,4,5])", numpy.array([2.1, 2.9]), np_cross1=[NDArray[float,:]])

    def test_cross2(self):
        self.run_test("def np_cross2(x): from numpy import cross ; return cross(x, -x)", numpy.array([2.1, 2.9, -2.1]), np_cross2=[NDArray[float,:]])

    def test_cross3(self):
        self.run_test("def np_cross3(x): from numpy import cross ; return cross(x, 2 * x)", numpy.array([[2.1, 2.9, -2.9]]), np_cross3=[NDArray[float,:, :]])

    def test_cross4(self):
        self.run_test("def np_cross4(x): from numpy import cross ; return cross(x, [[1,2]])", numpy.array([[2.9, -2.1, -2.9]]), np_cross4=[NDArray[float,:, :]])

    def test_finfo0(self):
        self.run_test("def np_finfo0(): from numpy import finfo, float64 ; x = finfo(float64) ; return x.eps", np_finfo0=[])

    def test_finfo1(self):
        self.run_test("def np_finfo1(x): from numpy import finfo ; f = finfo(x.dtype) ; return f.eps", numpy.ones(1), np_finfo1=[NDArray[float,:]])

    def test_finfo2(self):
        self.run_test("def np_finfo2(z): from numpy import finfo ; f = finfo(z.dtype) ; return f.eps < 1.0", numpy.array([1+1j]), np_finfo2=[NDArray[complex,:]])

    def test_fill0(self):
        self.run_test("def np_fill0(x): x.fill(5) ; return x", numpy.ones((2, 3)), np_fill0=[NDArray[float,:,:]])

    def test_eye0(self):
        self.run_test("def np_eye0(x): from numpy import eye ; return eye(x)", 2, np_eye0=[int])

    def test_eye1(self):
        self.run_test("def np_eye1(x): from numpy import eye ; return eye(x, x+1)", 2, np_eye1=[int])

    def test_eye1b(self):
        self.run_test("def np_eye1b(x): from numpy import eye ; return eye(x, x-1)", 3, np_eye1b=[int])

    def test_eye2(self):
        self.run_test("def np_eye2(x): from numpy import eye ; return eye(x, x, 1)", 2, np_eye2=[int])

    def test_eye3(self):
        self.run_test("def np_eye3(x): from numpy import eye, int32 ; return eye(x, x, 1, int32)", 2, np_eye3=[int])

    def test_eye4(self):
        self.run_test("def np_eye4(x): from numpy import eye, uint32 ; return eye(x, dtype=uint32)", 2, np_eye4=[int])

    def test_ediff1d0(self):
        self.run_test("def np_ediff1d0(x): from numpy import ediff1d ; return ediff1d(x)", [1,2,4,7,0], np_ediff1d0=[List[int]])

    def test_ediff1d1(self):
        self.run_test("def np_ediff1d1(x): from numpy import ediff1d ; return ediff1d(x)", [[1,2,4],[1,6,24]], np_ediff1d1=[List[List[int]]])

    def test_print_slice(self):
        self.run_test("def np_print_slice(a): print(a[:-1])", numpy.arange(12), np_print_slice=[NDArray[int,:]])

    def test_print_expr(self):
        self.run_test("def np_print_expr(a): print(a * 2)", numpy.arange(12), np_print_expr=[NDArray[int,:]])

    def test_broadcast_to0(self):
        self.run_test("def np_broadcast_to0(a, s): import numpy as np; return np.broadcast_to(a, s)", numpy.arange(12), (4, 12), np_broadcast_to0=[NDArray[int,:], Tuple[int, int]])

    def test_broadcast_to1(self):
        self.run_test("def np_broadcast_to1(a, s): import numpy as np; return np.broadcast_to(a, s)", numpy.arange(1), (4, 12), np_broadcast_to1=[NDArray[int,:], Tuple[int, int]])

    def test_broadcast_to2(self):
        self.run_test("def np_broadcast_to2(a, s): import numpy as np; return np.broadcast_to(a, s)", 5., (12, 2), np_broadcast_to2=[float, Tuple[int, int]])



def check_i_islarger2(i, last_i):
    islarger = i > last_i
    last_i = i
    return islarger, last_i

def omp_parallel_for_ordered():
    sum = 0
    is_larger = True
    last_i = 0

    'omp parallel for schedule(static, 1) ordered'
    for i in range(1,100):
        ii = i
        if 'omp ordered':
            tmp_is_larger, last_i = check_i_islarger2(i, last_i)
            is_larger = tmp_is_larger and is_larger
            sum += ii

    known_sum = (99 * 100) / 2
    return known_sum == sum and is_larger



#pythran export diffuseNumpy(float [][], float [][], int)
#runas import numpy as np;lx,ly=(2**7,2**7);u=np.zeros([lx,ly],dtype=np.double);u[lx//2,ly//2]=1000.0;tempU=np.zeros([lx,ly],dtype=np.double);diffuseNumpy(u,tempU,500)

import numpy as np


def diffuseNumpy(u, tempU, iterNum):
    """
    Apply Numpy matrix for the Forward-Euler Approximation
    """
    mu = .1

    for n in range(iterNum):
        tempU[1:-1, 1:-1] = u[1:-1, 1:-1] + mu * (
            u[2:, 1:-1] - 2 * u[1:-1, 1:-1] + u[0:-2, 1:-1] +
            u[1:-1, 2:] - 2 * u[1:-1, 1:-1] + u[1:-1, 0:-2])
        u[:, :] = tempU[:, :]
        tempU[:, :] = 0.0



#runas import numpy as np; n = 20; a = np.arange(n*n*n).reshape((n,n,n)).astype(np.uint8); b = 2. ; goodExpoMeasure(a, b)
#pythran export goodExpoMeasure(uint8[][][], float)
import numpy
def goodExpoMeasure(inRGB, sigma):
    '''
    Compute the good exposition image quality measure on 1 input image.
    '''
    R = inRGB[0,:,:].astype(numpy.float64)
    G = inRGB[1,:,:].astype(numpy.float64)
    B = inRGB[2,:,:].astype(numpy.float64)
    goodExpoR = numpy.exp(- ((R - 128)**2) / sigma)
    goodExpoG = numpy.exp(- ((G - 128)**2) / sigma)
    goodExpoB = numpy.exp(- ((B - 128)**2) / sigma)
    goodExpo  = goodExpoR * goodExpoG * goodExpoB
    goodExpo  = (numpy.round(goodExpo, 2) * (2**8-1)).astype(numpy.uint8)

    return goodExpo



#taken from http://eli.thegreenplace.net/2012/01/16/python-parallelizing-cpu-bound-tasks-with-multiprocessing/

#pythran export factorize_naive(int)
#runas factorize_naive(12222)
def factorize_naive(n):
    """ A naive factorization method. Take integer 'n', return list of
        factors.
    """
    if n < 2:
        return []
    factors = []
    p = 2

    while True:
        if n == 1:
            return factors

        r = n % p
        if r == 0:
            factors.append(p)
            n = n / p
        elif p * p >= n:
            factors.append(n)
            return factors
        elif p > 2:
            # Advance in steps of 2 over odd numbers
            p += 2
        else:
            # If p == 2, get to 3
            p += 1
    assert False, "unreachable"



#pythran export loopy(int list list, int, int, int)
#runas data = [[1, 45, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0], [0, 60, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0]] ; loopy(data, 0, 100, 100)
#skip.bench data = [[1, 45, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0]] + [[0, 60, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0]] * 200 ; loopy(data, 0, 100, 100) TOO_SLOW
def _WarningErrorHandler(msg,fatal, _WarningCount):
    if _WarningCount > 200:
        raise RuntimeError(msg)
    else:
        return _WarningCount +1

def loopy(_PopulationSetInfo_Data, _WarningCount, _NumberOfTriesToGenerateThisIndividual, _NumberOfTriesToGenerateThisSimulationStep):
    #### Functions Allowed in Expressions ####
    IndividualID = 0
    Repetition = 0
    Time = 0
    _ResultsInfo_Data = []
    #### Create State Handler Functions and State Classification Vector ##### 
    ############### Execute Simulation ###############
    ####### Subject Loop #######
    _Subject = 0
    while _Subject < (len(_PopulationSetInfo_Data)):
        IndividualID = IndividualID +1
        # Comment/Uncomment the next line to disable/enable printing of verbose information
        #print("Simulating Individual #" + str(IndividualID))
        _NumberOfTriesToGenerateThisIndividual = 1
        ##### Repetition Loop #####
        Repetition = 0
        while Repetition < (1000):
            # Reset repeat individual repetition flag in case it was set
            _RepeatSameIndividualRepetition = False
            #Init all parameters - Resetting them to zero
            # Comment/Uncomment the next line to disable/enable printing of verbose information
            #print("  Repetition = " + str(Repetition))
            Gender, Age, State0, State1, State2, State3Terminal, Example_6___Main_Process, Example_6___Main_Process_Entered, State0_Entered, State1_Entered, State2_Entered, State3Terminal_Entered = 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
            # Init parameters from population set
            [Gender, Age, State0, State1, State2, State3Terminal, Example_6___Main_Process, Example_6___Main_Process_Entered, State0_Entered, State1_Entered, State2_Entered, State3Terminal_Entered] = _PopulationSetInfo_Data[IndividualID-1]
            # Init parameters from Initialization Phase
            # Reset time and load first vector into results
            Time = 0
            # Load the initial condition into the results vector for this individual
            _ResultsInfoForThisIndividual = [ [IndividualID, Repetition, Time ,Gender, Age, State0, State1, State2, State3Terminal, Example_6___Main_Process, Example_6___Main_Process_Entered, State0_Entered, State1_Entered, State2_Entered, State3Terminal_Entered] ]
            _Terminate_Time_Loop = False or State3Terminal != 0
            _NumberOfTriesToGenerateThisSimulationStep = 0
            _RepeatSameSimulationStep = False
            ##### Time Loop #####
            while Time < 3:
                if _RepeatSameSimulationStep:
                    # if repeating the same simulation step, reset the flag to avoid infinite loops
                    _RepeatSameSimulationStep = False
                    # Load the previous time step results into the results vector for this individual
                    [_IgnoreIndividualID, _IgnoreRepetition, _IgnoreTime ,Gender, Age, State0, State1, State2, State3Terminal, Example_6___Main_Process, Example_6___Main_Process_Entered, State0_Entered, State1_Entered, State2_Entered, State3Terminal_Entered] = _ResultsInfoForThisIndividual[-1]
                    _Terminate_Time_Loop = False
                elif _Terminate_Time_Loop:
                    # If the time loop has to be terminated
                    break
                else:
                    # If not repeating the same simulation step, nor terminating, increase the time counter
                    Time = Time + 1
                # Comment/Uncomment the next line to disable/enable printing of verbose information
                #print("    Time Step = " + str(Time))
                # Reset Warning/Error Count
                _WarningCountBeforeThisSimulationStep = _WarningCount
                # Increase the number of Tries counter
                _NumberOfTriesToGenerateThisSimulationStep = _NumberOfTriesToGenerateThisSimulationStep + 1
                ##### Phase 1 - Pre State Transition #####
                # Processing the rule: "Affected Parameter: Age; Simulation Phase: Pre-stateOccurrence Probability: 1; Applied Formula: Age +1; Rule Notes: Age Increase; ; 
                _LastExpressionString = "Processing the expression: _Threshold = 1 ."
                # This expression should expand to: _Threshold = 1
                try:
                    # Building Step #0: _Threshold = 1
                    _Temp = 1
                    if not (-1e-14 <= _Temp  <= 1.00000000000001):
                        _WarningCount = _WarningErrorHandler("The occurrence probability threshold defined by a rule does not evaluate to a number between 0 and 1 within a tolerance specified by the system option parameter SystemPrecisionForProbabilityBoundCheck. The occurrence probability was evaluated to: " + str(_Temp) + " for the rule: " + 'Affected Parameter: Age; Simulation Phase: Pre-stateOccurrence Probability: 1; Applied Formula: Age +1; Rule Notes: Age Increase; ; ', True, _WarningCount) 
                except:
                    _WarningCount = _WarningErrorHandler(_LastExpressionString, True, _WarningCount)
                # Expression building complete - assign to destination parameter 
                _Threshold = _Temp
                if  0.5 < _Threshold:
                    _LastExpressionString = "Processing the expression: Age = Age +1 ."
                    # This expression should expand to: Age = Age +1
                    try:
                        # Building Step #0: Age = Age
                        _Temp0 = Age
                        # Building Step #1: Age = Age +1
                        _Temp = _Temp0 +1
                    except:
                        _WarningCount = _WarningErrorHandler(_LastExpressionString, True, _WarningCount)
                    # Expression building complete - assign to destination parameter 
                    Age = _Temp
                    pass
                ##### End of Rule Processing #####
                ##### Error Handlers #####
                if _WarningCount <= _WarningCountBeforeThisSimulationStep:
                    # Load New results to the results vector
                    _ResultsInfoForThisIndividual.append([IndividualID, Repetition, Time ,Gender, Age, State0, State1, State2, State3Terminal, Example_6___Main_Process, Example_6___Main_Process_Entered, State0_Entered, State1_Entered, State2_Entered, State3Terminal_Entered])
                    _NumberOfTriesToGenerateThisSimulationStep = 0
                else:
                    #print("    Repeating the same simulation step due to an error - probably a bad validity check")
                    _RepeatSameSimulationStep = True
                    if _NumberOfTriesToGenerateThisSimulationStep >= 5:
                        if _NumberOfTriesToGenerateThisIndividual < 2:
                            # Repeat the calculations for this person
                            _RepeatSameIndividualRepetition = True 
                            break
                        else:
                            _WarningCount = _WarningErrorHandler("The simulation was halted since the number of tries to recalculate the same person has been exceeded. If this problem consistently repeats itself, check the formulas to see if these cause too many out of bounds numbers to be generated. Alternatively, try raising the system option NumberOfTriesToRecalculateSimulationOfIndividualFromStart which is now defined as 2  .  ", True, _WarningCount)
            if _RepeatSameIndividualRepetition:
                #print("  Repeating the same repetition for the same individual due to exceeding the allowed number of simulation steps recalculations for this individual")
                _NumberOfTriesToGenerateThisIndividual = _NumberOfTriesToGenerateThisIndividual + 1
            else:
                # If going to the next individual repetition, save the results and increase the counter
                # Load New results to the results vector
                _ResultsInfo_Data.extend(_ResultsInfoForThisIndividual)
                Repetition = Repetition + 1
        _Subject = _Subject + 1
    # Comment/Uncomment the next lines to disable/enable dumping output file
    return _ResultsInfo_Data




def omp_in_parallel():
    import omp
    serial = 1
    isparallel = 0

    serial = omp.in_parallel()

    if 'omp parallel num_threads(2)':
        num_threads = omp.get_num_threads()
        if 'omp single':
            isparallel = omp.in_parallel()

    if 'omp parallel':
        if 'omp single':
            pass

    return bool(not serial and isparallel) or num_threads == 1



#runas solve()
#unittest.skip recursive generators
#pythran export solve()
'''
 A perfect number is a number for which the sum of its proper divisors is exactly equal to the number. For example, the sum of the proper divisors of 28 would be 1 + 2 + 4 + 7 + 14 = 28, which means that 28 is a perfect number.

 A number whose proper divisors are less than the number is called deficient and a number whose proper divisors exceed the number is called abundant.

 As 12 is the smallest abundant number, 1 + 2 + 3 + 4 + 6 = 16, the smallest number that can be written as the sum of two abundant numbers is 24. By mathematical analysis, it can be shown that all integers greater than 28123 can be written as the sum of two abundant numbers. However, this upper limit cannot be reduced any further by analysis even though it is known that the greatest number that cannot be expressed as the sum of two abundant numbers is less than this limit.

 Find the sum of all the positive integers which cannot be written as the sum of two abundant numbers.
 '''

def solve():
    prime_list = [2, 3, 5, 7, 11, 13, 17, 19, 23]   # Ensure that this is initialised with at least 1 prime
    prime_dict = dict.fromkeys(prime_list, 1)
    lastn      = prime_list[-1]
    def _isprime(n):
        ''' Raw check to see if n is prime. Assumes that prime_list is already populated '''
        isprime = n >= 2 and 1 or 0
        for prime in prime_list:                    # Check for factors with all primes
            if prime * prime > n: break             # ... up to sqrt(n)
            if not n % prime:
                isprime = 0
                break
        if isprime: prime_dict[n] = 1               # Maintain a dictionary for fast lookup
        return isprime

    def _refresh(x):
        ''' Refreshes primes upto x '''
        lastn = prime_list[-1]
        while lastn <= x:                           # Keep working until we've got up to x
            lastn = lastn + 1                       # Check the next number
            if _isprime(lastn):
                prime_list.append(lastn)            # Maintain a list for sequential access

    def factors(n):
        ''' Returns a prime factors of n as a list '''
        _refresh(n)
        x, xp, f = 0, prime_list[0], []
        while xp <= n:
            if not n % xp:
                f.append(xp)
                n = n / xp
            else:
                x = x + 1
                xp = prime_list[x]
        return f

    def all_factors(n):
        ''' Returns all factors of n, including 1 and n '''
        f = factors(n)
        elts = sorted(set(f))
        numelts = len(elts)
        def gen_inner(i):
            if i >= numelts:
                yield 1
                return
            thiselt = elts[i]
            thismax = f.count(thiselt)
            powers = [1]
            for j in range(thismax):
                powers.append(powers[-1] * thiselt)
            for d in gen_inner(i+1):
                for prime_power in powers:
                    yield prime_power * d
        for d in gen_inner(0):
            yield d

    MAX = 28124
    _refresh(MAX/2)
    abundants = [n for n in range(1, MAX) if sum(all_factors(n)) > n+n]
    abundants_dict = dict.fromkeys(abundants, 1)

    total = 0
    for n in range(1, MAX):
        sum_of_abundants = 0
        for a in abundants:
            if a > n: break
            if abundants_dict.get(n - a):
                sum_of_abundants = 1
                break
            if not sum_of_abundants:
                total = total + n

    return total



import numpy as np
#pythran export md(complex128[], complex128[])
#pythran export md(complex128[], complex128[::])
#runas import numpy as np;  s=np.arange(160.)+np.arange(160.)*1.j ; sc = s[::2]; md(s, sc)
def md(E, symbols):
    L = E.shape[0]
    M = symbols.shape[0]
    syms_out = np.zeros(L, dtype=E.dtype)
    #omp parallel for
    for i in range(L):
        im = np.argmin(abs(E[i]-symbols)**2)
        syms_out[i] = symbols[im]
    return syms_out



def omp_for_schedule_dynamic():
    CFDMAX_SIZE = 100
    chunk_size = 7
    tids = [0]*CFDMAX_SIZE
    count = 0
    tmp_count = 0
    result = 0
    import omp
    if 'omp parallel shared(tids)':
        tid = omp.get_thread_num()
        'omp for schedule(dynamic, chunk_size)'
        for i in range(CFDMAX_SIZE):
            tids[i] = tid

    for i in range(CFDMAX_SIZE-1):
        if tids[i] != tids[i+1]:
            count +=1

    tmp = [1] * (count + 1)
    for i in range(CFDMAX_SIZE-1):
        if tids[i] != tids[i+1]:
            tmp_count+=1
            tmp[tmp_count] = 1
        else:
            tmp[tmp_count]+=1

    for i in range(count):
        if tmp[i]%chunk_size != 0:
            result+=1
    if tmp[count]%chunk_size != CFDMAX_SIZE%chunk_size:
        result+=1
    return result == 0



#runas import numpy as np ; N = 10000 ; a = np.arange(float(N)); b = 0.1; c =1.1; log_likelihood(a, b, c)
#from http://arogozhnikov.github.io/2015/09/08/SpeedBenchmarks.html
import numpy

#pythran export log_likelihood(float64[], float64, float64)
def log_likelihood(data, mean, sigma):
    s = (data - mean) ** 2 / (2 * (sigma ** 2))
    pdfs = numpy.exp(- s)
    pdfs /= numpy.sqrt(2 * numpy.pi) * sigma
    return numpy.log(pdfs).sum()



#from http://rosettacode.org/wiki/Probabilistic_choice#Python
#pythran export test(str?, int?)
#runas test()

import random, bisect

def probchoice(items, probs, bincount=1000):
    '''
    Splits the interval 0.0-1.0 in proportion to probs
    then finds where each random.random() choice lies
    '''

    prob_accumulator = 0
    accumulator = []
    for p in probs:
        prob_accumulator += p
        accumulator.append(prob_accumulator)

    while True:
        r = random.random()
        yield items[bisect.bisect(accumulator, r)]

def probchoice2(items, probs, bincount=1000):
    '''
    Puts items in bins in proportion to probs
    then uses random.choice() to select items.

    Larger bincount for more memory use but
    higher accuracy (on avarage).
    '''

    bins = []
    for item,prob in zip(items, probs):
        bins += [item]*int(bincount*prob)
    while True:
        yield random.choice(bins)


def tester(func=probchoice, items=('good', 'bad' 'ugly'),
        probs=(0.5, 0.3, 0.2),
        trials = 100000
        ):
    def problist2string(probs):
        '''
        Turns a list of probabilities into a string
        Also rounds FP values
        '''
        return ",".join('%8.6f' % (p,) for p in probs)

    counter = dict()
    it = func(items, probs)
    for dummy in range(trials):
        k = next(it)
        if k in counter:
            counter[k] += 1
        else:
            counter[k] = 1
    print("\n##\n##\n##")
    print("Trials:              ", trials)
    print("Items:               ", ' '.join(items))
    print("Target probability:  ", problist2string(probs))
    print("Attained probability:", problist2string(
        counter[x]/float(trials) for x in items))

def test(init_seq='aleph beth gimel daleth he waw zayin heth', bincount=1000000):
    items = init_seq.split()
    probs = [1/(float(n)+5) for n in range(len(items))]
    probs[-1] = 1-sum(probs[:-1])
    tester(probchoice, items, probs, bincount)
    tester(probchoice2, items, probs, 1000000)



#from http://rosettacode.org/wiki/Ramsey%27s_theorem#Python
#pythran export test()
#runas test()

def test():
    range17 =  range(17)
    a = [['0'] * 17 for i in range17]
    for i in range17:
        a[i][i] = '-'
    for k in range(4):
        for i in range17:
            j = (i + pow(2, k)) % 17
            a[i][j] = a[j][i] = '1'
    for row in a:
        print(' '.join(row))



from distutils.core import setup, Extension
from pythran.dist import PythranExtension, PythranBuildExt

setup(name = 'demo2',
      version = '1.0',
      description = 'This is another demo package',
      packages = ['demo2'],
      cmdclass={"build_ext": PythranBuildExt},
      ext_modules = [PythranExtension('demo2.a', sources = ['a.py'])])



#pythran export solve(int)
#runas solve(3)
def solve(digit):
    '''
    A palindromic number reads the same both ways. The largest palindrome made from the product of two 2-digit numbers is 9009 = 91 x 99.
    
    Find the largest palindrome made from the product of two 3-digit numbers.
    '''

    n = 0
    for a in range(10 ** digit - 1, 10 ** (digit - 1), -1):
        for b in range(a, 10 ** (digit - 1), -1):
            x = a * b
            if x > n:
                s = str(a * b)
                if s == s[::-1]:
                    n = a * b

    return n



#pythran export dist(float [], float[], float[], int, bool, bool, bool)
#runas import numpy as np ; N = 20 ; x = np.arange(0., N, 0.1) ; L = 4 ; periodic = True ; dist(x, x, x, L,periodic, periodic, periodic)
#bench import numpy as np ; N = 300 ; x = np.arange(0., N, 0.1) ; L = 4 ; periodic = True ; dist(x, x, x, L,periodic, periodic, periodic)
import numpy as np

def dist(x, y, z, L, periodicX, periodicY, periodicZ):
    " ""Computes distances between all particles and places the result in a matrix such that the ij th matrix entry corresponds to the distance between particle i and j"" "
    N = len(x)
    xtemp = np.tile(x,(N,1))
    dx = xtemp - xtemp.T
    ytemp = np.tile(y,(N,1))
    dy = ytemp - ytemp.T
    ztemp = np.tile(z,(N,1))
    dz = ztemp - ztemp.T

    # Particles 'feel' each other across the periodic boundaries
    if periodicX:
        dx[dx>L/2]=dx[dx > L/2]-L
        dx[dx<-L/2]=dx[dx < -L/2]+L

    if periodicY:
        dy[dy>L/2]=dy[dy>L/2]-L
        dy[dy<-L/2]=dy[dy<-L/2]+L

    if periodicZ:
        dz[dz>L/2]=dz[dz>L/2]-L
        dz[dz<-L/2]=dz[dz<-L/2]+L

    # Total Distances
    d = np.sqrt(dx**2+dy**2+dz**2)

    # Mark zero entries with negative 1 to avoid divergences
    d[d==0] = -1

    return d, dx, dy, dz



def omp_single_nowait():
    total_iterations = 0
    nr_iterations = 0
    LOOPCOUNT = 1000
    i = 0

    if 'omp parallel private(i)':
        for i in range(LOOPCOUNT):
            if 'omp single nowait':
                'omp atomic'
                nr_iterations += 1

    if 'omp parallel private(i)':
        my_iterations = 0
        for i in range(LOOPCOUNT):
            if 'omp single nowait':
                my_iterations += 1
        if 'omp critical':
            total_iterations += my_iterations

    return nr_iterations == LOOPCOUNT and total_iterations == LOOPCOUNT



import omp
from time import sleep


def omp_flush():
    result1 = 0
    result2 = 0
    if 'omp parallel':
        use_omp = omp.in_parallel()
        rank = omp.get_thread_num()
        'omp barrier'
        if rank == 1:
            result2 = 3
            'omp flush (result2)'
            dummy = result2
        if rank == 0:
            sleep(0.5)
            'omp flush(result2)'
            result1 = result2
    return not use_omp or (result1 == result2 and result2 == dummy and result2 == 3)



def omp_master_3():
    import omp
    tid_result = 0
    nthreads = 0
    executing_thread = -1

    if 'omp parallel':
        if 'omp master':
            tid = omp.get_thread_num()
            if tid != 0:
                if 'omp critical':
                    tid_result += 1
            if 'omp critical':
                nthreads += 1
            executing_thread = omp.get_thread_num()
    return nthreads == 1 and executing_thread == 0 and tid_result == 0



# cython: np_pythran=True
import numpy as np
cimport numpy as cnp

def test():
    """
    >>> test()
    """
    lx, ly = (2**7, 2**7)
    u = np.zeros([lx, ly], dtype=np.double)
    u[lx // 2, ly // 2] = 1000.0
    diffuse_numpy(u, 500)
    return u


def diffuse_numpy(cnp.ndarray[double, ndim=2] u, int N):
    """
    Apply Numpy matrix for the Forward-Euler Approximation
    """
    cdef cnp.ndarray[double, ndim=2] temp = np.zeros_like(u)
    mu = 0.1

    for n in range(N):
        temp[1:-1, 1:-1] = u[1:-1, 1:-1] + mu * (
            u[2:, 1:-1] - 2L * u[1:-1, 1:-1] + u[0:-2, 1:-1] +
            u[1:-1, 2:] - 2L * u[1:-1, 1:-1] + u[1:-1, 0:-2])
        u[:, :] = temp[:, :]
        temp[:, :] = 0.0



# from https://github.com/sklam/numba-example-wavephysics
#runas test(50)
#bench test(55000)
import numpy as np
from math import ceil

def physics(masspoints, dt, plunk, which):
  ppos = masspoints[1]
  cpos = masspoints[0]
  N = cpos.shape[0]
  # apply hooke's law
  HOOKE_K = 2100000.
  DAMPING = 0.0001
  MASS = .01

  force = np.zeros((N, 2))
  for i in range(1, N):
    dx, dy = cpos[i] - cpos[i - 1]
    dist = np.sqrt(dx**2 + dy**2)
    assert dist != 0
    fmag = -HOOKE_K * dist
    cosine = dx / dist
    sine = dy / dist
    fvec = np.array([fmag * cosine, fmag * sine])
    force[i - 1] -= fvec
    force[i] += fvec

  force[0] = force[-1] = 0, 0
  force[which][1] += plunk
  accel = force / MASS

  # verlet integration
  npos = (2 - DAMPING) * cpos - (1 - DAMPING) * ppos + accel * (dt**2)

  masspoints[1] = cpos
  masspoints[0] = npos

#pythran export test(int)
def test(PARTICLE_COUNT):
    SUBDIVISION = 300
    FRAMERATE = 60
    count = PARTICLE_COUNT
    width, height = 600, 200

    masspoints = np.empty((2, count, 2), np.float64)
    initpos = np.zeros(count, np.float64)
    for i in range(1, count):
        initpos[i] = initpos[i - 1] + float(width) / count
    masspoints[:, :, 0] = initpos
    masspoints[:, :, 1] = height / 2
    f = 15
    plunk_pos = count // 2
    physics( masspoints, 1./ (SUBDIVISION * FRAMERATE), f, plunk_pos)
    return masspoints[0, count // 2]



from pythran.typing import List, NDArray
from pythran.tests import TestFromDir, TestEnv

import os
import numpy as np
import platform

# from http://www.scipy.org/Download , weave/example directory

class TestScipy(TestFromDir):

    def test_laplace(self):
        code="""
def laplace(u,dx, dy):
    nx, ny=len(u), len(u[0])
    for i in range(1, nx-1):
        for j in range(1, ny-1):
            u[i][j] = ((u[i-1][j] + u[i+1][j])*dy**2 +
                      (u[i][j-1] + u[i][j+1])*dx**2)/(2.0*(dx**2 + dy**2))
"""
        self.run_test(code, [[0.1,0.2,0.3],[0.1,0.2,0.3],[0.1,0.2,0.3]], 0.01, 0.02, laplace=[List[List[float]], float, float])

    def test_recursive_fibonnaci(self):
        code="""
def recursive_fibonnaci(a):
    if a <= 2:
        return 1
    else:
        return recursive_fibonnaci(a-2) + recursive_fibonnaci(a-1)
        """
        self.run_test(code, 5, recursive_fibonnaci=[int])

    def test_iterative_fibonnaci(self):
        code="""
def iterative_fibonnaci(a):
    if a <= 2:
        return 1
    last = next_to_last = 1
    i = 2
    while i < a:
        result = last + next_to_last
        next_to_last = last
        last = result
        i+=1
    return result;
"""
        self.run_test(code, 5, iterative_fibonnaci=[int])

    def test_binary_search(self):
        code="""
def binary_search(seq, t):
    min = 0; max = len(seq) - 1
    while 1:
        if max < min:
            return -1
        m = (min + max) // 2
        if seq[m] < t:
            min = m + 1
        elif seq[m] > t:
            max = m - 1
        else:
            return m
"""
        self.run_test(code,[1,2,3,4,5,6,7,8,9], 4, binary_search=[List[int], int])

    def test_ramp(self):
        code="""
def ramp(result, start, end):
    size=len(result)
    assert size > 1
    step = (end-start)//(size-1)
    for i in range(size):
        result[i] = start + step*i
"""
        self.run_test(code,[0. for x in range(10)], 1.5, 9.5, ramp=[List[float], float, float])

    path = os.path.join(os.path.dirname(__file__), "scipy")

TestScipy.populate(TestScipy)

class TestPyData(TestFromDir):
    path = os.path.join(os.path.dirname(__file__), "pydata")

TestPyData.populate(TestPyData)

class TestScikitImage(TestFromDir):
    path = os.path.join(os.path.dirname(__file__), "scikit-image")

TestPyData.populate(TestScikitImage)

try:
    import scipy

    if not platform.machine().startswith('ppc64'):

        @TestEnv.module
        class TestScipySpecial(TestEnv):
            def test_jv_scalar(self):
                self.run_test("""
                from scipy.special import jv
                def jv_scalar(v, x):
                    return jv(v, x)""",
                    5, 1.414,
                    jv_scalar=[int, float])
            def test_spherical_jn_scalar(self):
                self.run_test("""
                from scipy.special import spherical_jn
                def spherical_bessel_j_scalar(v, x):
                    return spherical_jn(v, x)""",
                              5, 1.414,
                              spherical_bessel_j_scalar=[int, float])

            def test_spherical_jn_arg1d(self):
                self.run_test("""
                from scipy.special import spherical_jn
                def spherical_bessel_j_1d(v, x):
                    return spherical_jn(v, x)""",
                              5, np.array([1.0, 2.0, 3.0]),
                              spherical_bessel_j_1d=[int, NDArray[float,:]])

            def test_spherical_jn_order1d(self):
                self.run_test("""
                from scipy.special import spherical_jn
                def spherical_bessel_j_order1d(v, x):
                    return spherical_jn(v, x)""",
                              np.array([1, 2, 3]), 5.0,
                              spherical_bessel_j_order1d=[NDArray[int,:], float])

            def test_spherical_jn_arg2d(self):
                self.run_test("""
                from scipy.special import spherical_jn
                def spherical_bessel_j_2d(v, x):
                    return spherical_jn(v, x)""",
                              5, np.array([[1.0, 2.0], [3.0, 4.0]]),
                              spherical_bessel_j_2d=[int, NDArray[float,:,:]])

            def test_binom_scalar(self):
                self.run_test("""
                from scipy.special import binom
                def binom_scalar(v, x):
                    return binom(v, x)""",
                              5, 4,
                              binom_scalar=[int, int])

            def test_binom_arg1d(self):
                self.run_test("""
                from scipy.special import binom
                def binom_1d(v, x):
                    return binom(v, x)""",
                              5, np.array([1, 2, 3]),
                              binom_1d=[int, NDArray[int,:]])

except ImportError:
    pass



#pythran export deriv(int, float, float, complex list, complex list, complex list, float list list, float)

import cmath

#This subroutine finds a root of a polynomial of degree n > 2
#        by Laguerre's method.
def guerre(a,n,z,err,nter):
    az = [complex(0,0) for i in range(50)]
    azz = [complex(0,0) for i in range(50)]
    ci=complex(0.0,1.0)
    eps=1.0e-20
#  The coefficients of p'[z] and p''[z].
    for i in range(1,n+1):
        az[i-1]=float(i)*a[i]
    for i in range(1,n):
        azz[i-1]=float(i)*az[i]
    dz=err+1
    itera=0
    jter=0
    while abs(dz)>err and itera<nter:
        p=a[n-1]+a[n]*z
        for i in range(n-1,0,-1):
            p=a[i-1]+z*p
        if abs(p)<eps:
            return z
        pz=az[n-2]+az[n-1]*z
        for i in range(n-2,0,-1):
            pz=az[i-1]+z*pz
        pzz=azz[n-3]+azz[n-2]*z
        for i in range(n-3,0,-1):
            pzz=azz[i-1]+z*pzz
#  The Laguerre perturbation.
        f=pz/p
        g=(f**2)-pzz/p
        h= n*g#cmath.sqrt((float(n)-1.0)*(n*g-(f**2)))
        amp1=abs(f+h);
        amp2=abs(f-h);
        if amp1>amp2:
            dz=float(-n)/(f+h)
        else:
            dz=float(-n)/(f-h)
        itera=itera+1
#   Rotate by 90 degrees to avoid limit cycles. 
        jter=jter+1
        if jter==10:
            jter=1
            dz=dz*ci
        z=z+dz
        if jter==100:
            raise RuntimeError("Laguerre method not converging")
    return z

#   The root-finding subroutine. 
def fndrt(a,n):
    z=[complex(0,0) for k in range(n) ]
    if n==1:
        z[0]=-a[0]/a[1]
        return z
    if n>2:
        for k in range(n,2,-1):
#   Obtain an approximate root.
            root=complex(0.0,0)
            err=1.0e-12
            root = guerre(a,k,root,err,1000)
#   Refine the root by iterating five more times.
            err=0.0;
            root = guerre(a,k,root,err,5)
            z[k-1]=root
#   Divide out the factor [z-root].
            for i in range(k,0,-1):
                a[i-1]=a[i-1]+root*a[i]
            for i in range(1,k+1):
                a[i-1]=a[i];
#   Solve the quadratic equation.
    z[1]=0.5*(-a[1]+cmath.sqrt(a[1]*a[1]-4.0*a[0]*a[2]))/a[2]
    z[0]=0.5*(-a[1]-cmath.sqrt(a[1]*a[1]-4.0*a[0]*a[2]))/a[2]
    return z

#   Rows are interchanged for stability.
def pivot(n,i,a,b):
    i0=i
    amp0=abs(a[i-1][i-1])
    for j in range(i+1,n+1):
        amp=abs(a[i-1][j-1])
        if amp>amp0:
            i0=j
            amp0=amp
    if i==i0:
        return
    temp=b[i-1]
    b[i-1]=b[i0-1];
    b[i0-1]=temp;
    for j in range(i,n+1):
        temp=a[j-1][i-1]
        a[j-1][i-1]=a[j-1][i0-1]
        a[j-1][i0-1]=temp

def gauss(n,a,b):
#     Downward elimination.
    for i in range(1,n+1):
        if i<n:
            pivot(n,i,a,b)
        a[i-1][i-1]=1.0/a[i-1][i-1]
        b[i-1]=b[i-1]*a[i-1][i-1]
        if i<n:
            for j in range(i+1,n+1):
                a[j-1][i-1]=a[j-1][i-1]*a[i-1][i-1]
            for k in range(i+1,n+1):
                b[k-1]=b[k-1]-a[i-1][k-1]*b[i-1]
                for j in range(i+1,n+1):
                    a[j-1][k-1]=a[j-1][k-1]-a[i-1][k-1]*a[j-1][i-1]
#     Back substitution.
    for i in range(n-1,0,-1):
        for j in range(i,n):
            b[i-1]=b[i-1]-a[j][i-1]*b[j]

#     The derivatives of the operator function at x=0.
def deriv(n,sig,alp,dg,dh1,dh3,bin,nu):
    dh2=[complex(0,0) for n in dh1]
    ci = complex(0.0,1.0)
    dh1[0]=complex(0.5,0)*ci*complex(sig,0)
    exp1 = complex(-0.5,0)
    dh2[0]=alp;
    exp2 = complex(-1.0,0)
    dh3[0]=-2.0*nu
    exp3 = complex(-1.0,0)
    for i in range(1,n):
        dh1[i]=dh1[i-1]*exp1
        exp1=exp1-1.0
        dh2[i]=dh2[i-1]*exp2
        exp2=exp2-1.0
        dh3[i]=-nu*dh3[i-1]*exp3
        exp3=exp3-1.0
    dg[0]=1.0
    dg[1]=dh1[0]+dh2[0]+dh3[0]
    for i in range(2,n+1):
        dg[i]=dh1[i-1]+dh2[i-1]+dh3[i-1]
        for j in range(1,i):
            dg[i]=dg[i]+bin[j-1][i-1]*(dh1[j-1]+dh2[j-1]+dh3[j-1])*dg[i-j]

#     The coefficients of the rational approximation.
def epade(np,ns,ip,k0,dr,pd1,pd2):
    m=40
    dg = range(m)
    dh1 = range(m)
    dh3 = range(m)
    a = [[ 0 for i in range(m)] for j in range(m)]
    b = range(m);
    bin = [[ 0 for i in range(m)] for j in range(m)]
    fact = range(m)
    ci=complex(0.0,1.0)
    sig=k0*dr
    n=2*np
    if ip==1:
        nu=0.0
        alp=0.0
    else:
        nu=1.0
        alp=-0.25
#     The factorials
    fact[0]=1.0
    for i in range(1,n):
        fact[i]=(i+1)*fact[i-1]
#     The binomial coefficients.;
    for i in range(0,n+1):
        bin[0][i]=1.0
        bin[i][i]=1.0
    for i in range(2,n+1):
        for j in range(1,i):
            bin[j][i]=bin[j-1][i-1]+bin[j][i-1]
    for i in range(0,n):
        for j in range(0,n):
             a[i][j]=0.0
#     The accuracy constraints.;
    deriv(n, sig, alp, dg, dh1, dh3, bin, nu)
    for i in range(0,n):
        b[i]=dg[i+1]
    for i in range(1,n+1):
        if 2*i-1<=n:
            a[2*i-2][i-1]=fact[i-1]
        for j in range(1,i+1):
            if 2*j<=n:
                a[2*j-1][i-1]=-bin[j][i]*fact[j-1]*dg[i-j]
#     The stability constraints.;
    if ns>=1:
        z1=-3.0
        b[n-1]=-1.0
        for j in range(1,np+1): 
            a[2*j-2][n-1]=z1 ** j
            a[2*j-1][n-1]=0.0
    if ns>=2:
        z1=-1.5
        b[n-2]=-1.0
        for j in range(1,np+1):
            a[2*j-2][n-2]=z1 ** j
            a[2*j-1][n-2]=0.0
    gauss(n,a,b)
    dh1[0]=1.0
    for j in range(1,np+1):
        dh1[j]=b[2*j-2]
    dh2=fndrt(dh1,np)
    for j in range(0,np):
        pd1[j]=-1.0/dh2[j]
    dh1[0]=1.0
    for j in range(1,np+1):
        dh1[j]=b[2*j-1]
    dh2=fndrt(dh1,np)
    for j in range(0,np):
        pd2[j]=-1.0/dh2[j]

#     The tridiagonal matrices.
def matrc(nz,np,iz,dz,k0,rhob,alpw,alpb,ksq,ksqw,ksqb,f1,f2,f3,r1,r2,r3,s1,s2,s3,pd1,pd2,izsrf):
    a1=k0*k0/6.0
    a2=2.0*k0*k0/3.0
    a3=k0*k0/6.0
    cfact=0.5/(dz*dz)
    dfact=1.0/12.0
    for i in range(0,iz):
        f1[i]=1.0/alpw[i]
        f2[i]=1.0
        f3[i]=alpw[i]
        ksq[i]=ksqw[i]
    ii=0
    for i in range(iz,nz+2):
        f1[i]=rhob[ii]/alpb[ii]
        f2[i]=1.0/rhob[ii]
        f3[i]=alpb[ii]
        ksq[i]=ksqb[ii]
        ii+=1
    for i in range(1,nz+1):
#     Discretization by Galerkin's method.
        c1=cfact*f1[i]*(f2[i-1]+f2[i])*f3[i-1]
        c2=-cfact*f1[i]*(f2[i-1]+2.0*f2[i]+f2[i+1])*f3[i]
        c3=cfact*f1[i]*(f2[i]+f2[i+1])*f3[i+1]
        d1=c1+dfact*(ksq[i-1]+ksq[i])
        d2=c2+dfact*(ksq[i-1]+complex(6.0,0)*ksq[i]+ksq[i+1])
        d3=c3+dfact*(ksq[i]+ksq[i+1])
        for j in range(0,np):
            r1[j][i]=a1+pd2[j]*d1
            r2[j][i]=a2+pd2[j]*d2
            r3[j][i]=a3+pd2[j]*d3
            s1[j][i]=a1+pd1[j]*d1
            s2[j][i]=a2+pd1[j]*d2
            s3[j][i]=a3+pd1[j]*d3
#     The entries above the surface.
    for j in range(0,np):
        for i in range(0,izsrf):
            r1[j][i]=0.0
            r2[j][i]=1.0
            r3[j][i]=0.0
            s1[j][i]=0.0
            s2[j][i]=0.0
            s3[j][i]=0.0
#     The matrix decomposition.
    for j in range(0,np):
        for i in range(1,nz+1):
            rfact=complex(1.0,0)/(r2[j][i]-r1[j][i]*r3[j][i-1])
            r1[j][i]=r1[j][i]*rfact
            r3[j][i]=r3[j][i]*rfact
            s1[j][i]=s1[j][i]*rfact
            s2[j][i]=s2[j][i]*rfact
            s3[j][i]=s3[j][i]*rfact

##     Matrix updates.
#def updat(fs1,nz,np,iz,ib,dr,dz,eta,omega,rmax,c0,k0,ci,r,rp,rs,rb,zb,cw,cb,rhob,attn, \
#alpw,alpb,ksq,ksqw,ksqb,f1,f2,f3,r1,r2,r3,s1,s2,s3,pd1,pd2,rsrf,zsrf,izsrf,isrf,attw):
##     Varying bathymetry.
#    if r>=rb[ib]:
#        ib=ib+1
#    if r>=rsrf[isrf]:
#        isrf=isrf+1
#    jzsrf=izsrf
#    z=zsrf[isrf-1]+(r+0.5*dr-rsrf[isrf-1])*(zsrf[isrf]-zsrf[isrf-1])/(rsrf[isrf]-rsrf[isrf-1])
#    izsrf=int(z/dz)
#    jz=iz
#    z=zb[ib-1]+(r+0.5*dr-rb[ib-1])*(zb[ib]-zb[ib-1])/(rb[ib]-rb[ib-1])
#    iz=int(1.0+z/dz)
#    iz=max(2,iz)
#    iz=min(nz,iz)
#    if iz!=jz or izsrf != jzsrf:
#        matrc(nz,np,iz,dz,k0,rhob,alpw,alpb,ksq,ksqw,ksqb,f1,f2,f3,r1,r2,r3,s1,s2,s3,pd1,pd2,izsrf)
##     Varying profiles.
#    if r>=rp:
#        rp = profl(fs1,nz,ci,dz,eta,omega,rmax,c0,k0,rp,cw,cb,rhob,attn,alpw,alpb,ksqw,ksqb,attw)
#        matrc(nz,np,iz,dz,k0,rhob,alpw,alpb,ksq,ksqw,ksqb,f1,f2,f3,r1,r2,r3,s1,s2,s3,pd1,pd2,izsrf)
##     Turn off the stability constraints.
#    if r>=rs:
#        ns=0
#        epade(np,ns,1,k0,dr,pd1,pd2)
#        matrc(nz,np,iz,dz,k0,rhob,alpw,alpb,ksq,ksqw,ksqb,f1,f2,f3,r1,r2,r3,s1,s2,s3,pd1,pd2,izsrf)
#    return ib,isrf,izsrf,iz,rp



#runas solve()
#unittest.skip date time not supported
#pythran export solve()
def solve():
    '''
    You are given the following information, but you may prefer to do some research for yourself.

    1 Jan 1900 was a Monday.
    Thirty days has September,
    April, June and November.
    All the rest have thirty-one,
    Saving February alone,
    Which has twenty-eight, rain or shine.
    And on leap years, twenty-nine.
    A leap year occurs on any year evenly divisible by 4, but not on a century unless it is divisible by 400.

    How many Sundays fell on the first of the month during the twentieth century (1 Jan 1901 to 31 Dec 2000)?
    '''
    import datetime

    sundays = 0
    for year in range(1901, 2001):
        for month in range(1, 13):
            d = datetime.date(year, month, 1)
            if d.weekday() == 6:
                sundays = sundays + 1

    return sundays




#runas solve(10000)
#pythran export solve(int)
'''
 Let d(n) be defined as the sum of proper divisors of n (numbers less than n which divide evenly into n).
 If d(a) = b and d(b) = a, where a  b, then a and b are an amicable pair and each of a and b are called amicable numbers.

 For example, the proper divisors of 220 are 1, 2, 4, 5, 10, 11, 20, 22, 44, 55 and 110; therefore d(220) = 284. The proper divisors of 284 are 1, 2, 4, 71 and 142; so d(284) = 220.

 Evaluate the sum of all the amicable numbers under 10000.
'''

def divisors(n): return list(i for i in range(1, n//2+1) if n % i == 0)
def solve(m):
	pair = dict( ((n, sum(divisors(n))) for n in range(1, m)) )
	return sum(n for n in range(1, m) if pair.get(pair[n], 0) == n and pair[n] != n)



def omp_sections_lastprivate():
    sum =0
    sum0 = 0
    i0 = -1
    if 'omp parallel':
        if 'omp sections private(sum0) lastprivate(i0)':
            if 'omp section':
                sum0 = 0
                for i in range(1, 400):
                    sum0 += i
                    i0 = i
                if 'omp critical':
                    sum += sum0
            if 'omp section':
                sum0 = 0
                for i in range(400, 700):
                    sum0 += i
                    i0 = i
                if 'omp critical':
                    sum += sum0
            if 'omp section':
                sum0 = 0
                for i in range(700, 1000):
                    sum0 += i
                    i0 = i
                if 'omp critical':
                    sum += sum0

    known_sum = (999 * 1000) / 2
    return known_sum == sum and i0 == 999



def omp_nested():
    import omp
    counter = 0
    omp.set_nested(1)

    if 'omp parallel shared(counter) num_threads(4)':
        use_omp = omp.in_parallel()
        if 'omp critical':
            counter += 1

        if 'omp parallel num_threads(4)':
            if 'omp critical':
                counter -= 1
    return counter != 0 or not use_omp



#runas solve()
#pythran export solve()
def solve():
    '''
    It was proposed by Christian Goldbach that every odd composite number can be written as the sum of a prime and twice a square.
    
     9 =  7 + 2 x 1^2
    15 =  7 + 2 x 2^2
    21 =  3 + 2 x 3^2
    25 =  7 + 2 x 3^2
    27 = 19 + 2 x 2^2
    33 = 31 + 2 x 1^2
    It turns out that the conjecture was false.
    
    What is the smallest odd composite that cannot be written as the sum of a prime and twice a square?
    '''

    prime_list = [2, 3, 5, 7, 11, 13, 17, 19, 23]   # Ensure that this is initialised with at least 1 prime
    prime_dict = dict.fromkeys(prime_list, 1)

    def _isprime(n):
        ''' Raw check to see if n is prime. Assumes that prime_list is already populated '''
        isprime = n >= 2 and 1 or 0
        for prime in prime_list:                    # Check for factors with all primes
            if prime * prime > n: break             # ... up to sqrt(n)
            if not n % prime:
                isprime = 0
                break
        if isprime: prime_dict[n] = 1               # Maintain a dictionary for fast lookup
        return isprime

    def _refresh(x):
        ''' Refreshes primes upto x '''
        lastn = prime_list[-1]
        while lastn <= x:                           # Keep working until we've got up to x
            lastn = lastn + 1                       # Check the next number
            if _isprime(lastn):
                prime_list.append(lastn)            # Maintain a list for sequential access

    def prime(x):
        ''' Returns the xth prime '''

        lastn = prime_list[-1]
        while len(prime_list) <= x:                 # Keep working until we've got the xth prime
            lastn = lastn + 1                       # Check the next number
            if _isprime(lastn):
                prime_list.append(lastn)            # Maintain a list for sequential access
        return prime_list[x]

    MAX = 10000
    squares = dict.fromkeys((x*x for x in range(1, MAX)), 1)
    _refresh(MAX)

    for x in range(35, MAX, 2):
        if not _isprime(x):
            is_goldbach = 0
            for p in prime_list[1:]:
                if p >= x: break
                if ((x - p)/2) in squares:
                    is_goldbach = 1
                    break
            if not is_goldbach:
                return x
                break




#from http://rosettacode.org/wiki/Read_a_specific_line_from_a_file#Python
#pythran export readline(int)
#runas readline(6)

def readline(n):
    from itertools import islice

    f = open('pythran/tests/rosetta/read_conf.cfg')
    linelist = list(islice(f, n, n + 1))
    assert linelist != [], 'Not ' + str(n) + ' lines in file'
    line = linelist[0]
    f.close()
    return line



#from: http://stackoverflow.com/questions/20799403/improving-performance-of-cronbach-alpha-code-python-numpy
#pythran export cronbach(float [][])
#runas import numpy as np ; N = 800 ; items = np.arange(N*N, dtype=float).reshape(N,N) ; cronbach(items)
#bench import numpy as np ; N = 800 ; items = np.arange(N*N, dtype=float).reshape(N,N) ; cronbach(items)
def cronbach(itemscores):
    itemvars = itemscores.var(1, None, None, 1)#(axis=1, ddof=1)
    tscores = itemscores.sum(0)#(axis=0)
    nitems = len(itemscores)
    return nitems / (nitems-1) * (1 - itemvars.sum() / tscores.var(None, None, None, 1))#(ddof=1))



import unittest
from pythran.tests import TestEnv
import numpy
from pythran.typing import NDArray

try:
    import scipy
except ImportError:
    scipy = None


@TestEnv.module
class TestNumpyUFuncUnary(TestEnv):
    pass


# automatic generation of basic test cases for ufunc
unary_func_by_module = {
    'numpy': (
        'abs', 'absolute', 'arccos', 'arccosh', 'arcsin', 'arcsinh', 'arctan',
        'arctanh',
        'bitwise_not',
        'cbrt', 'ceil', 'conj', 'conjugate', 'cos', 'cosh',
        'deg2rad', 'degrees',
        'exp', 'expm1',
        'fabs', 'float32', 'float64', 'floor',
        'int8', 'int16', 'int32', 'int64', 'isinf', 'isneginf', 'isposinf',
        'isnan', 'invert', 'isfinite',
        'log', 'log10', 'log1p', 'log2', 'logical_not',
        'negative',
        'rad2deg', 'radians', 'reciprocal', 'rint', 'round', 'round_',
        'sign', 'signbit',
        'sin', 'sinh', 'spacing', 'sqrt', 'square',
        'tan', 'tanh', 'trunc',
        'uint8', 'uint16', 'uint32', 'uint64'
    ),
    'scipy.special': ('gammaln', 'gamma', 'i0', 'i0e')
}

if scipy is None:
    del unary_func_by_module['scipy.special']

test_inputs_by_type = {
    'float': {
        '_scalar': ('0.5', '[float]'),
        '': ('numpy.ones(10)', '[NDArray[float,:]]'),
        '_matrix': ('numpy.ones((2,5))', '[NDArray[float,:,:]]')
    },
    'complex': {
        '_scalar': ('0.5j', '[complex]'),
        '': ('numpy.ones(10)*1.j', '[NDArray[complex,:]]'),
        '_matrix': ('numpy.ones((2,5))*1.j', '[NDArray[complex,:,:]]')
    },
    'numpy.int32': {
        '_scalar': ('numpy.int32(1)', '[numpy.int32]'),
        '': ('numpy.ones(10, numpy.int32)', '[NDArray[numpy.int32,:]]'),
        '_matrix': ('numpy.ones((2,5), numpy.int32)',
                    '[NDArray[numpy.int32,:,:]]')
    }
}

for module, functions in unary_func_by_module.items():
    for f in functions:
        for input_type in ('float', 'complex'):
            try:
                eval('{}.{}({})'.format(module, f, test_inputs_by_type[input_type]['_scalar'][0]))
            except TypeError:
                if input_type == 'float':
                    input_type = 'numpy.int32'
                else:
                    continue  # no need to test that, it does not work on numpy

            if input_type == 'complex' and 'gamma' in f:
                continue  # skip


            for test_suffix, (input, pythran_input_type) \
                    in test_inputs_by_type[input_type].items():

                func_name = "numpy_ufunc_unary_{}_{}{}_{}".format(module.replace('.', '_'), f,
                                             test_suffix, input_type.replace('.','_'))

                code = """
                    lambda self: self.run_test(
                        '''{func}''', 
                        {input}, 
                        {func_name}={pythran_input_type}
                    )
                    """.replace('\n', '').format(
                        func="""
                        def {func_name}(a):
                            from {module} import {f}
                            return {f}(a)
                        """.format(func_name=func_name, module=module, f=f),
                        func_name=func_name,
                        input=input,
                        pythran_input_type=pythran_input_type,
                    )

                setattr(
                    TestNumpyUFuncUnary,
                    'test_numpy_ufunc_unary_{}'.format(func_name),
                    eval(code)
                )
